/*
 * Automatically generated C config: don't edit
 */
#define AUTOCONF_INCLUDED
#define CONFIG_X86 1
#undef  CONFIG_SBUS
#define CONFIG_UID16 1

/*
 * Code maturity level options
 */
#define CONFIG_EXPERIMENTAL 1

/*
 * Loadable module support
 */
#define CONFIG_MODULES 1
#undef  CONFIG_MODVERSIONS
#define CONFIG_KMOD 1

/*
 * Processor type and features
 */
#undef  CONFIG_M386
#define CONFIG_M486 1
#undef  CONFIG_M586
#undef  CONFIG_M586TSC
#undef  CONFIG_M586MMX
#undef  CONFIG_M686
#undef  CONFIG_MPENTIUMIII
#undef  CONFIG_MPENTIUM4
#undef  CONFIG_MK6
#undef  CONFIG_MK7
#undef  CONFIG_MK8
#undef  CONFIG_MELAN
#undef  CONFIG_MCRUSOE
#undef  CONFIG_MWINCHIPC6
#undef  CONFIG_MWINCHIP2
#undef  CONFIG_MWINCHIP3D
#undef  CONFIG_MCYRIXIII
#undef  CONFIG_MVIAC3_2
#define CONFIG_X86_WP_WORKS_OK 1
#define CONFIG_X86_INVLPG 1
#define CONFIG_X86_CMPXCHG 1
#define CONFIG_X86_XADD 1
#define CONFIG_X86_BSWAP 1
#define CONFIG_X86_POPAD_OK 1
#undef  CONFIG_RWSEM_GENERIC_SPINLOCK
#define CONFIG_RWSEM_XCHGADD_ALGORITHM 1
#define CONFIG_X86_L1_CACHE_SHIFT (4)
#define CONFIG_X86_USE_STRING_486 1
#define CONFIG_X86_ALIGNMENT_16 1
#define CONFIG_X86_PPRO_FENCE 1
#undef  CONFIG_X86_F00F_WORKS_OK
#undef  CONFIG_X86_MCE
#undef  CONFIG_TOSHIBA
#define CONFIG_TOSHIBA_MODULE 1
#undef  CONFIG_I8K
#define CONFIG_I8K_MODULE 1
#undef  CONFIG_MICROCODE
#define CONFIG_MICROCODE_MODULE 1
#undef  CONFIG_X86_MSR
#define CONFIG_X86_MSR_MODULE 1
#undef  CONFIG_X86_CPUID
#define CONFIG_X86_CPUID_MODULE 1
#undef  CONFIG_EDD
#define CONFIG_EDD_MODULE 1
#define CONFIG_NOHIGHMEM 1
#undef  CONFIG_HIGHMEM4G
#undef  CONFIG_HIGHMEM64G
#undef  CONFIG_HIGHMEM
#define CONFIG_MATH_EMULATION 1
#define CONFIG_MTRR 1
#undef  CONFIG_SMP
#undef  CONFIG_X86_UP_APIC
#undef  CONFIG_X86_UP_IOAPIC
#undef  CONFIG_X86_TSC_DISABLE

/*
 * General setup
 */
#define CONFIG_NET 1
#define CONFIG_PCI 1
#undef  CONFIG_PCI_GOBIOS
#undef  CONFIG_PCI_GODIRECT
#define CONFIG_PCI_GOANY 1
#define CONFIG_PCI_BIOS 1
#define CONFIG_PCI_DIRECT 1
#define CONFIG_ISA 1
#define CONFIG_PCI_NAMES 1
#undef  CONFIG_EISA
#undef  CONFIG_MCA
#define CONFIG_HOTPLUG 1

/*
 * PCMCIA/CardBus support
 */
#undef  CONFIG_PCMCIA
#define CONFIG_PCMCIA_MODULE 1
#define CONFIG_CARDBUS 1
#define CONFIG_TCIC 1
#define CONFIG_I82092 1
#define CONFIG_I82365 1

/*
 * PCI Hotplug Support
 */
#undef  CONFIG_HOTPLUG_PCI
#define CONFIG_HOTPLUG_PCI_MODULE 1
#undef  CONFIG_HOTPLUG_PCI_COMPAQ
#define CONFIG_HOTPLUG_PCI_COMPAQ_MODULE 1
#undef  CONFIG_HOTPLUG_PCI_COMPAQ_NVRAM
#define CONFIG_SYSVIPC 1
#define CONFIG_BSD_PROCESS_ACCT 1
#define CONFIG_SYSCTL 1
#define CONFIG_KCORE_ELF 1
#undef  CONFIG_KCORE_AOUT
#undef  CONFIG_BINFMT_AOUT
#define CONFIG_BINFMT_AOUT_MODULE 1
#define CONFIG_BINFMT_ELF 1
#undef  CONFIG_BINFMT_MISC
#define CONFIG_BINFMT_MISC_MODULE 1
#undef  CONFIG_OOM_KILLER
#define CONFIG_PM 1
#undef  CONFIG_APM
#define CONFIG_APM_MODULE 1
#undef  CONFIG_APM_IGNORE_USER_SUSPEND
#undef  CONFIG_APM_DO_ENABLE
#undef  CONFIG_APM_CPU_IDLE
#undef  CONFIG_APM_DISPLAY_BLANK
#undef  CONFIG_APM_RTC_IS_GMT
#undef  CONFIG_APM_ALLOW_INTS
#undef  CONFIG_APM_REAL_MODE_POWER_OFF

/*
 * ACPI Support
 */
#undef  CONFIG_ACPI

/*
 * Memory Technology Devices (MTD)
 */
#undef  CONFIG_MTD

/*
 * Parallel port support
 */
#undef  CONFIG_PARPORT
#define CONFIG_PARPORT_MODULE 1
#undef  CONFIG_PARPORT_PC
#define CONFIG_PARPORT_PC_MODULE 1
#undef  CONFIG_PARPORT_PC_CML1
#define CONFIG_PARPORT_PC_CML1_MODULE 1
#undef  CONFIG_PARPORT_SERIAL
#define CONFIG_PARPORT_SERIAL_MODULE 1
#undef  CONFIG_PARPORT_PC_FIFO
#undef  CONFIG_PARPORT_PC_SUPERIO
#undef  CONFIG_PARPORT_PC_PCMCIA
#define CONFIG_PARPORT_PC_PCMCIA_MODULE 1
#undef  CONFIG_PARPORT_AMIGA
#undef  CONFIG_PARPORT_MFC3
#undef  CONFIG_PARPORT_ATARI
#undef  CONFIG_PARPORT_GSC
#undef  CONFIG_PARPORT_SUNBPP
#undef  CONFIG_PARPORT_IP22
#undef  CONFIG_PARPORT_OTHER
#define CONFIG_PARPORT_1284 1

/*
 * Plug and Play configuration
 */
#undef  CONFIG_PNP
#define CONFIG_PNP_MODULE 1
#undef  CONFIG_ISAPNP
#define CONFIG_ISAPNP_MODULE 1

/*
 * Block devices
 */
#define CONFIG_BLK_DEV_FD 1
#undef  CONFIG_BLK_DEV_XD
#define CONFIG_BLK_DEV_XD_MODULE 1
#undef  CONFIG_PARIDE
#define CONFIG_PARIDE_MODULE 1
#undef  CONFIG_PARIDE_PARPORT
#define CONFIG_PARIDE_PARPORT_MODULE 1

/*
 * Parallel IDE high-level drivers
 */
#undef  CONFIG_PARIDE_PD
#define CONFIG_PARIDE_PD_MODULE 1
#undef  CONFIG_PARIDE_PCD
#define CONFIG_PARIDE_PCD_MODULE 1
#undef  CONFIG_PARIDE_PF
#define CONFIG_PARIDE_PF_MODULE 1
#undef  CONFIG_PARIDE_PT
#define CONFIG_PARIDE_PT_MODULE 1
#undef  CONFIG_PARIDE_PG
#define CONFIG_PARIDE_PG_MODULE 1

/*
 * Parallel IDE protocol modules
 */
#undef  CONFIG_PARIDE_ATEN
#define CONFIG_PARIDE_ATEN_MODULE 1
#undef  CONFIG_PARIDE_BPCK
#define CONFIG_PARIDE_BPCK_MODULE 1
#undef  CONFIG_PARIDE_BPCK6
#define CONFIG_PARIDE_BPCK6_MODULE 1
#undef  CONFIG_PARIDE_COMM
#define CONFIG_PARIDE_COMM_MODULE 1
#undef  CONFIG_PARIDE_DSTR
#define CONFIG_PARIDE_DSTR_MODULE 1
#undef  CONFIG_PARIDE_FIT2
#define CONFIG_PARIDE_FIT2_MODULE 1
#undef  CONFIG_PARIDE_FIT3
#define CONFIG_PARIDE_FIT3_MODULE 1
#undef  CONFIG_PARIDE_EPAT
#define CONFIG_PARIDE_EPAT_MODULE 1
#define CONFIG_PARIDE_EPATC8 1
#undef  CONFIG_PARIDE_EPIA
#define CONFIG_PARIDE_EPIA_MODULE 1
#undef  CONFIG_PARIDE_FRIQ
#define CONFIG_PARIDE_FRIQ_MODULE 1
#undef  CONFIG_PARIDE_FRPW
#define CONFIG_PARIDE_FRPW_MODULE 1
#undef  CONFIG_PARIDE_KBIC
#define CONFIG_PARIDE_KBIC_MODULE 1
#undef  CONFIG_PARIDE_KTTI
#define CONFIG_PARIDE_KTTI_MODULE 1
#undef  CONFIG_PARIDE_ON20
#define CONFIG_PARIDE_ON20_MODULE 1
#undef  CONFIG_PARIDE_ON26
#define CONFIG_PARIDE_ON26_MODULE 1
#undef  CONFIG_BLK_CPQ_DA
#define CONFIG_BLK_CPQ_DA_MODULE 1
#undef  CONFIG_BLK_CPQ_CISS_DA
#define CONFIG_BLK_CPQ_CISS_DA_MODULE 1
#define CONFIG_CISS_SCSI_TAPE 1
#undef  CONFIG_CISS_MONITOR_THREAD
#undef  CONFIG_BLK_DEV_DAC960
#define CONFIG_BLK_DEV_DAC960_MODULE 1
#undef  CONFIG_BLK_DEV_UMEM
#define CONFIG_BLK_DEV_UMEM_MODULE 1
#define CONFIG_BLK_DEV_LOOP 1
#undef  CONFIG_BLK_DEV_NBD
#define CONFIG_BLK_DEV_NBD_MODULE 1
#define CONFIG_BLK_DEV_RAM 1
#define CONFIG_BLK_DEV_RAM_SIZE (7777)
#define CONFIG_BLK_DEV_INITRD 1
#undef  CONFIG_BLK_STATS

/*
 * Multi-device support (RAID and LVM)
 */
#define CONFIG_MD 1
#define CONFIG_BLK_DEV_MD 1
#define CONFIG_MD_LINEAR 1
#define CONFIG_MD_RAID0 1
#define CONFIG_MD_RAID1 1
#define CONFIG_MD_RAID5 1
#undef  CONFIG_MD_MULTIPATH
#define CONFIG_MD_MULTIPATH_MODULE 1
#define CONFIG_BLK_DEV_LVM 1

/*
 * Networking options
 */
#define CONFIG_PACKET 1
#define CONFIG_PACKET_MMAP 1
#undef  CONFIG_NETLINK_DEV
#define CONFIG_NETLINK_DEV_MODULE 1
#define CONFIG_NETFILTER 1
#undef  CONFIG_NETFILTER_DEBUG
#define CONFIG_FILTER 1
#define CONFIG_UNIX 1
#define CONFIG_INET 1
#define CONFIG_IP_MULTICAST 1
#define CONFIG_IP_ADVANCED_ROUTER 1
#define CONFIG_IP_MULTIPLE_TABLES 1
#define CONFIG_IP_ROUTE_FWMARK 1
#define CONFIG_IP_ROUTE_NAT 1
#define CONFIG_IP_ROUTE_MULTIPATH 1
#define CONFIG_IP_ROUTE_TOS 1
#define CONFIG_IP_ROUTE_VERBOSE 1
#undef  CONFIG_IP_PNP
#undef  CONFIG_NET_IPIP
#define CONFIG_NET_IPIP_MODULE 1
#undef  CONFIG_NET_IPGRE
#define CONFIG_NET_IPGRE_MODULE 1
#define CONFIG_NET_IPGRE_BROADCAST 1
#define CONFIG_IP_MROUTE 1
#define CONFIG_IP_PIMSM_V1 1
#undef  CONFIG_IP_PIMSM_V2
#undef  CONFIG_ARPD
#undef  CONFIG_INET_ECN
#define CONFIG_SYN_COOKIES 1

/*
 *   IP: Netfilter Configuration
 */
#undef  CONFIG_IP_NF_CONNTRACK
#define CONFIG_IP_NF_CONNTRACK_MODULE 1
#undef  CONFIG_IP_NF_FTP
#define CONFIG_IP_NF_FTP_MODULE 1
#undef  CONFIG_IP_NF_AMANDA
#define CONFIG_IP_NF_AMANDA_MODULE 1
#undef  CONFIG_IP_NF_TFTP
#define CONFIG_IP_NF_TFTP_MODULE 1
#undef  CONFIG_IP_NF_IRC
#define CONFIG_IP_NF_IRC_MODULE 1
#undef  CONFIG_IP_NF_QUEUE
#define CONFIG_IP_NF_QUEUE_MODULE 1
#undef  CONFIG_IP_NF_IPTABLES
#define CONFIG_IP_NF_IPTABLES_MODULE 1
#undef  CONFIG_IP_NF_MATCH_LIMIT
#define CONFIG_IP_NF_MATCH_LIMIT_MODULE 1
#undef  CONFIG_IP_NF_MATCH_MAC
#define CONFIG_IP_NF_MATCH_MAC_MODULE 1
#undef  CONFIG_IP_NF_MATCH_PKTTYPE
#define CONFIG_IP_NF_MATCH_PKTTYPE_MODULE 1
#undef  CONFIG_IP_NF_MATCH_MARK
#define CONFIG_IP_NF_MATCH_MARK_MODULE 1
#undef  CONFIG_IP_NF_MATCH_MULTIPORT
#define CONFIG_IP_NF_MATCH_MULTIPORT_MODULE 1
#undef  CONFIG_IP_NF_MATCH_TOS
#define CONFIG_IP_NF_MATCH_TOS_MODULE 1
#undef  CONFIG_IP_NF_MATCH_RECENT
#define CONFIG_IP_NF_MATCH_RECENT_MODULE 1
#undef  CONFIG_IP_NF_MATCH_ECN
#define CONFIG_IP_NF_MATCH_ECN_MODULE 1
#undef  CONFIG_IP_NF_MATCH_DSCP
#define CONFIG_IP_NF_MATCH_DSCP_MODULE 1
#undef  CONFIG_IP_NF_MATCH_AH_ESP
#define CONFIG_IP_NF_MATCH_AH_ESP_MODULE 1
#undef  CONFIG_IP_NF_MATCH_LENGTH
#define CONFIG_IP_NF_MATCH_LENGTH_MODULE 1
#undef  CONFIG_IP_NF_MATCH_TTL
#define CONFIG_IP_NF_MATCH_TTL_MODULE 1
#undef  CONFIG_IP_NF_MATCH_TCPMSS
#define CONFIG_IP_NF_MATCH_TCPMSS_MODULE 1
#undef  CONFIG_IP_NF_MATCH_HELPER
#define CONFIG_IP_NF_MATCH_HELPER_MODULE 1
#undef  CONFIG_IP_NF_MATCH_STATE
#define CONFIG_IP_NF_MATCH_STATE_MODULE 1
#undef  CONFIG_IP_NF_MATCH_CONNTRACK
#define CONFIG_IP_NF_MATCH_CONNTRACK_MODULE 1
#undef  CONFIG_IP_NF_MATCH_UNCLEAN
#define CONFIG_IP_NF_MATCH_UNCLEAN_MODULE 1
#undef  CONFIG_IP_NF_MATCH_OWNER
#define CONFIG_IP_NF_MATCH_OWNER_MODULE 1
#undef  CONFIG_IP_NF_FILTER
#define CONFIG_IP_NF_FILTER_MODULE 1
#undef  CONFIG_IP_NF_TARGET_REJECT
#define CONFIG_IP_NF_TARGET_REJECT_MODULE 1
#undef  CONFIG_IP_NF_TARGET_MIRROR
#define CONFIG_IP_NF_TARGET_MIRROR_MODULE 1
#undef  CONFIG_IP_NF_NAT
#define CONFIG_IP_NF_NAT_MODULE 1
#define CONFIG_IP_NF_NAT_NEEDED 1
#undef  CONFIG_IP_NF_TARGET_MASQUERADE
#define CONFIG_IP_NF_TARGET_MASQUERADE_MODULE 1
#undef  CONFIG_IP_NF_TARGET_REDIRECT
#define CONFIG_IP_NF_TARGET_REDIRECT_MODULE 1
#undef  CONFIG_IP_NF_NAT_AMANDA
#define CONFIG_IP_NF_NAT_AMANDA_MODULE 1
#undef  CONFIG_IP_NF_NAT_LOCAL
#undef  CONFIG_IP_NF_NAT_SNMP_BASIC
#define CONFIG_IP_NF_NAT_SNMP_BASIC_MODULE 1
#undef  CONFIG_IP_NF_NAT_IRC
#define CONFIG_IP_NF_NAT_IRC_MODULE 1
#undef  CONFIG_IP_NF_NAT_FTP
#define CONFIG_IP_NF_NAT_FTP_MODULE 1
#undef  CONFIG_IP_NF_NAT_TFTP
#define CONFIG_IP_NF_NAT_TFTP_MODULE 1
#undef  CONFIG_IP_NF_MANGLE
#define CONFIG_IP_NF_MANGLE_MODULE 1
#undef  CONFIG_IP_NF_TARGET_TOS
#define CONFIG_IP_NF_TARGET_TOS_MODULE 1
#undef  CONFIG_IP_NF_TARGET_ECN
#define CONFIG_IP_NF_TARGET_ECN_MODULE 1
#undef  CONFIG_IP_NF_TARGET_DSCP
#define CONFIG_IP_NF_TARGET_DSCP_MODULE 1
#undef  CONFIG_IP_NF_TARGET_MARK
#define CONFIG_IP_NF_TARGET_MARK_MODULE 1
#undef  CONFIG_IP_NF_TARGET_LOG
#define CONFIG_IP_NF_TARGET_LOG_MODULE 1
#undef  CONFIG_IP_NF_TARGET_ULOG
#define CONFIG_IP_NF_TARGET_ULOG_MODULE 1
#undef  CONFIG_IP_NF_TARGET_TCPMSS
#define CONFIG_IP_NF_TARGET_TCPMSS_MODULE 1
#undef  CONFIG_IP_NF_ARPTABLES
#define CONFIG_IP_NF_ARPTABLES_MODULE 1
#undef  CONFIG_IP_NF_ARPFILTER
#define CONFIG_IP_NF_ARPFILTER_MODULE 1
#undef  CONFIG_IP_NF_ARP_MANGLE
#define CONFIG_IP_NF_ARP_MANGLE_MODULE 1
#undef  CONFIG_IP_NF_COMPAT_IPCHAINS
#define CONFIG_IP_NF_COMPAT_IPCHAINS_MODULE 1
#define CONFIG_IP_NF_NAT_NEEDED 1
#undef  CONFIG_IP_NF_COMPAT_IPFWADM

/*
 *   IP: Virtual Server Configuration
 */
#undef  CONFIG_IP_VS
#define CONFIG_IP_VS_MODULE 1
#undef  CONFIG_IP_VS_DEBUG
#define CONFIG_IP_VS_TAB_BITS (12)

/*
 * IPVS scheduler
 */
#undef  CONFIG_IP_VS_RR
#define CONFIG_IP_VS_RR_MODULE 1
#undef  CONFIG_IP_VS_WRR
#define CONFIG_IP_VS_WRR_MODULE 1
#undef  CONFIG_IP_VS_LC
#define CONFIG_IP_VS_LC_MODULE 1
#undef  CONFIG_IP_VS_WLC
#define CONFIG_IP_VS_WLC_MODULE 1
#undef  CONFIG_IP_VS_LBLC
#define CONFIG_IP_VS_LBLC_MODULE 1
#undef  CONFIG_IP_VS_LBLCR
#define CONFIG_IP_VS_LBLCR_MODULE 1
#undef  CONFIG_IP_VS_DH
#define CONFIG_IP_VS_DH_MODULE 1
#undef  CONFIG_IP_VS_SH
#define CONFIG_IP_VS_SH_MODULE 1
#undef  CONFIG_IP_VS_SED
#define CONFIG_IP_VS_SED_MODULE 1
#undef  CONFIG_IP_VS_NQ
#define CONFIG_IP_VS_NQ_MODULE 1

/*
 * IPVS application helper
 */
#undef  CONFIG_IP_VS_FTP
#define CONFIG_IP_VS_FTP_MODULE 1
#undef  CONFIG_IPV6
#define CONFIG_IPV6_MODULE 1

/*
 *   IPv6: Netfilter Configuration
 */
#undef  CONFIG_IP6_NF_QUEUE
#undef  CONFIG_IP6_NF_IPTABLES
#define CONFIG_IP6_NF_IPTABLES_MODULE 1
#undef  CONFIG_IP6_NF_MATCH_LIMIT
#define CONFIG_IP6_NF_MATCH_LIMIT_MODULE 1
#undef  CONFIG_IP6_NF_MATCH_MAC
#define CONFIG_IP6_NF_MATCH_MAC_MODULE 1
#undef  CONFIG_IP6_NF_MATCH_RT
#define CONFIG_IP6_NF_MATCH_RT_MODULE 1
#undef  CONFIG_IP6_NF_MATCH_OPTS
#define CONFIG_IP6_NF_MATCH_OPTS_MODULE 1
#undef  CONFIG_IP6_NF_MATCH_FRAG
#define CONFIG_IP6_NF_MATCH_FRAG_MODULE 1
#undef  CONFIG_IP6_NF_MATCH_HL
#define CONFIG_IP6_NF_MATCH_HL_MODULE 1
#undef  CONFIG_IP6_NF_MATCH_MULTIPORT
#define CONFIG_IP6_NF_MATCH_MULTIPORT_MODULE 1
#undef  CONFIG_IP6_NF_MATCH_OWNER
#define CONFIG_IP6_NF_MATCH_OWNER_MODULE 1
#undef  CONFIG_IP6_NF_MATCH_MARK
#define CONFIG_IP6_NF_MATCH_MARK_MODULE 1
#undef  CONFIG_IP6_NF_MATCH_IPV6HEADER
#define CONFIG_IP6_NF_MATCH_IPV6HEADER_MODULE 1
#undef  CONFIG_IP6_NF_MATCH_AHESP
#define CONFIG_IP6_NF_MATCH_AHESP_MODULE 1
#undef  CONFIG_IP6_NF_MATCH_LENGTH
#define CONFIG_IP6_NF_MATCH_LENGTH_MODULE 1
#undef  CONFIG_IP6_NF_MATCH_EUI64
#define CONFIG_IP6_NF_MATCH_EUI64_MODULE 1
#undef  CONFIG_IP6_NF_FILTER
#define CONFIG_IP6_NF_FILTER_MODULE 1
#undef  CONFIG_IP6_NF_TARGET_LOG
#define CONFIG_IP6_NF_TARGET_LOG_MODULE 1
#undef  CONFIG_IP6_NF_MANGLE
#define CONFIG_IP6_NF_MANGLE_MODULE 1
#undef  CONFIG_IP6_NF_TARGET_MARK
#define CONFIG_IP6_NF_TARGET_MARK_MODULE 1
#undef  CONFIG_KHTTPD
#define CONFIG_KHTTPD_MODULE 1

/*
 *    SCTP Configuration (EXPERIMENTAL)
 */
#undef  CONFIG_IP_SCTP
#define CONFIG_IP_SCTP_MODULE 1
#undef  CONFIG_SCTP_DBG_MSG
#undef  CONFIG_SCTP_DBG_OBJCNT
#undef  CONFIG_SCTP_HMAC_NONE
#undef  CONFIG_SCTP_HMAC_SHA1
#define CONFIG_SCTP_HMAC_MD5 1
#undef  CONFIG_ATM
#undef  CONFIG_VLAN_8021Q
#define CONFIG_VLAN_8021Q_MODULE 1

/*
 *  
 */
#undef  CONFIG_IPX
#define CONFIG_IPX_MODULE 1
#undef  CONFIG_IPX_INTERN
#undef  CONFIG_ATALK
#define CONFIG_ATALK_MODULE 1

/*
 * Appletalk devices
 */
#undef  CONFIG_DEV_APPLETALK
#undef  CONFIG_DECNET
#define CONFIG_DECNET_MODULE 1
#undef  CONFIG_DECNET_SIOCGIFCONF
#undef  CONFIG_DECNET_ROUTER
#undef  CONFIG_BRIDGE
#define CONFIG_BRIDGE_MODULE 1
#undef  CONFIG_X25
#define CONFIG_X25_MODULE 1
#undef  CONFIG_LAPB
#define CONFIG_LAPB_MODULE 1
#undef  CONFIG_LLC
#undef  CONFIG_NET_DIVERT
#undef  CONFIG_ECONET
#undef  CONFIG_WAN_ROUTER
#define CONFIG_WAN_ROUTER_MODULE 1
#undef  CONFIG_NET_FASTROUTE
#undef  CONFIG_NET_HW_FLOWCONTROL

/*
 * QoS and/or fair queueing
 */
#define CONFIG_NET_SCHED 1
#undef  CONFIG_NET_SCH_CBQ
#define CONFIG_NET_SCH_CBQ_MODULE 1
#undef  CONFIG_NET_SCH_HTB
#define CONFIG_NET_SCH_HTB_MODULE 1
#undef  CONFIG_NET_SCH_CSZ
#define CONFIG_NET_SCH_CSZ_MODULE 1
#undef  CONFIG_NET_SCH_HFSC
#define CONFIG_NET_SCH_HFSC_MODULE 1
#undef  CONFIG_NET_SCH_PRIO
#define CONFIG_NET_SCH_PRIO_MODULE 1
#undef  CONFIG_NET_SCH_RED
#define CONFIG_NET_SCH_RED_MODULE 1
#undef  CONFIG_NET_SCH_SFQ
#define CONFIG_NET_SCH_SFQ_MODULE 1
#undef  CONFIG_NET_SCH_TEQL
#define CONFIG_NET_SCH_TEQL_MODULE 1
#undef  CONFIG_NET_SCH_TBF
#define CONFIG_NET_SCH_TBF_MODULE 1
#undef  CONFIG_NET_SCH_GRED
#define CONFIG_NET_SCH_GRED_MODULE 1
#undef  CONFIG_NET_SCH_DELAY
#define CONFIG_NET_SCH_DELAY_MODULE 1
#undef  CONFIG_NET_SCH_DSMARK
#define CONFIG_NET_SCH_DSMARK_MODULE 1
#undef  CONFIG_NET_SCH_INGRESS
#define CONFIG_NET_SCH_INGRESS_MODULE 1
#define CONFIG_NET_QOS 1
#define CONFIG_NET_ESTIMATOR 1
#define CONFIG_NET_CLS 1
#undef  CONFIG_NET_CLS_TCINDEX
#define CONFIG_NET_CLS_TCINDEX_MODULE 1
#undef  CONFIG_NET_CLS_ROUTE4
#define CONFIG_NET_CLS_ROUTE4_MODULE 1
#define CONFIG_NET_CLS_ROUTE 1
#undef  CONFIG_NET_CLS_FW
#define CONFIG_NET_CLS_FW_MODULE 1
#undef  CONFIG_NET_CLS_U32
#define CONFIG_NET_CLS_U32_MODULE 1
#undef  CONFIG_NET_CLS_RSVP
#define CONFIG_NET_CLS_RSVP_MODULE 1
#undef  CONFIG_NET_CLS_RSVP6
#define CONFIG_NET_CLS_RSVP6_MODULE 1
#define CONFIG_NET_CLS_POLICE 1

/*
 * Network testing
 */
#undef  CONFIG_NET_PKTGEN
#define CONFIG_NET_PKTGEN_MODULE 1

/*
 * Telephony Support
 */
#undef  CONFIG_PHONE
#define CONFIG_PHONE_MODULE 1
#undef  CONFIG_PHONE_IXJ
#define CONFIG_PHONE_IXJ_MODULE 1
#undef  CONFIG_PHONE_IXJ_PCMCIA
#define CONFIG_PHONE_IXJ_PCMCIA_MODULE 1

/*
 * ATA/IDE/MFM/RLL support
 */
#define CONFIG_IDE 1

/*
 * IDE, ATA and ATAPI Block devices
 */
#define CONFIG_BLK_DEV_IDE 1

/*
 * Please see Documentation/ide.txt for help/info on IDE drives
 */
#undef  CONFIG_BLK_DEV_HD_IDE
#undef  CONFIG_BLK_DEV_HD
#define CONFIG_BLK_DEV_IDEDISK 1
#undef  CONFIG_IDEDISK_MULTI_MODE
#undef  CONFIG_IDEDISK_STROKE
#undef  CONFIG_BLK_DEV_IDECS
#define CONFIG_BLK_DEV_IDECS_MODULE 1
#define CONFIG_BLK_DEV_IDECD 1
#undef  CONFIG_BLK_DEV_IDETAPE
#define CONFIG_BLK_DEV_IDETAPE_MODULE 1
#define CONFIG_BLK_DEV_IDEFLOPPY 1
#undef  CONFIG_BLK_DEV_IDESCSI
#define CONFIG_BLK_DEV_IDESCSI_MODULE 1
#undef  CONFIG_IDE_TASK_IOCTL

/*
 * IDE chipset support/bugfixes
 */
#define CONFIG_BLK_DEV_CMD640 1
#undef  CONFIG_BLK_DEV_CMD640_ENHANCED
#undef  CONFIG_BLK_DEV_ISAPNP
#define CONFIG_BLK_DEV_IDEPCI 1
#define CONFIG_BLK_DEV_GENERIC 1
#define CONFIG_IDEPCI_SHARE_IRQ 1
#define CONFIG_BLK_DEV_IDEDMA_PCI 1
#undef  CONFIG_BLK_DEV_OFFBOARD
#undef  CONFIG_BLK_DEV_IDEDMA_FORCED
#define CONFIG_IDEDMA_PCI_AUTO 1
#undef  CONFIG_IDEDMA_ONLYDISK
#define CONFIG_BLK_DEV_IDEDMA 1
#undef  CONFIG_IDEDMA_PCI_WIP
#define CONFIG_BLK_DEV_ADMA100 1
#define CONFIG_BLK_DEV_AEC62XX 1
#define CONFIG_BLK_DEV_ALI15X3 1
#undef  CONFIG_WDC_ALI15X3
#define CONFIG_BLK_DEV_AMD74XX 1
#undef  CONFIG_AMD74XX_OVERRIDE
#define CONFIG_BLK_DEV_ATIIXP 1
#define CONFIG_BLK_DEV_CMD64X 1
#define CONFIG_BLK_DEV_TRIFLEX 1
#undef  CONFIG_BLK_DEV_CY82C693
#define CONFIG_BLK_DEV_CS5530 1
#define CONFIG_BLK_DEV_HPT34X 1
#undef  CONFIG_HPT34X_AUTODMA
#define CONFIG_BLK_DEV_HPT366 1
#define CONFIG_BLK_DEV_PIIX 1
#undef  CONFIG_BLK_DEV_NS87415
#undef  CONFIG_BLK_DEV_OPTI621
#define CONFIG_BLK_DEV_PDC202XX_OLD 1
#undef  CONFIG_PDC202XX_BURST
#define CONFIG_BLK_DEV_PDC202XX_NEW 1
#undef  CONFIG_PDC202XX_FORCE
#define CONFIG_BLK_DEV_RZ1000 1
#define CONFIG_BLK_DEV_SC1200 1
#define CONFIG_BLK_DEV_SVWKS 1
#define CONFIG_BLK_DEV_SIIMAGE 1
#define CONFIG_BLK_DEV_SIS5513 1
#define CONFIG_BLK_DEV_SLC90E66 1
#undef  CONFIG_BLK_DEV_TRM290
#define CONFIG_BLK_DEV_VIA82CXXX 1
#define CONFIG_IDE_CHIPSETS 1

/*
 * Note: most of these also require special kernel boot parameters
 */
#define CONFIG_BLK_DEV_4DRIVES 1
#define CONFIG_BLK_DEV_ALI14XX 1
#define CONFIG_BLK_DEV_DTC2278 1
#define CONFIG_BLK_DEV_HT6560B 1
#define CONFIG_BLK_DEV_PDC4030 1
#define CONFIG_BLK_DEV_QD65XX 1
#define CONFIG_BLK_DEV_UMC8672 1
#define CONFIG_IDEDMA_AUTO 1
#undef  CONFIG_IDEDMA_IVB
#undef  CONFIG_DMA_NONPCI
#define CONFIG_BLK_DEV_PDC202XX 1
#undef  CONFIG_BLK_DEV_ATARAID
#define CONFIG_BLK_DEV_ATARAID_MODULE 1
#undef  CONFIG_BLK_DEV_ATARAID_PDC
#define CONFIG_BLK_DEV_ATARAID_PDC_MODULE 1
#undef  CONFIG_BLK_DEV_ATARAID_HPT
#define CONFIG_BLK_DEV_ATARAID_HPT_MODULE 1
#undef  CONFIG_BLK_DEV_ATARAID_MEDLEY
#define CONFIG_BLK_DEV_ATARAID_MEDLEY_MODULE 1
#undef  CONFIG_BLK_DEV_ATARAID_SII
#define CONFIG_BLK_DEV_ATARAID_SII_MODULE 1

/*
 * SCSI support
 */
#define CONFIG_SCSI 1

/*
 * SCSI support type (disk, tape, CD-ROM)
 */
#define CONFIG_BLK_DEV_SD 1
#define CONFIG_SD_EXTRA_DEVS (40)
#undef  CONFIG_CHR_DEV_ST
#define CONFIG_CHR_DEV_ST_MODULE 1
#undef  CONFIG_CHR_DEV_OSST
#define CONFIG_CHR_DEV_OSST_MODULE 1
#define CONFIG_BLK_DEV_SR 1
#undef  CONFIG_BLK_DEV_SR_VENDOR
#define CONFIG_SR_EXTRA_DEVS (2)
#define CONFIG_CHR_DEV_SG 1

/*
 * Some SCSI devices (e.g. CD jukebox) support multiple LUNs
 */
#define CONFIG_SCSI_DEBUG_QUEUES 1
#undef  CONFIG_SCSI_MULTI_LUN
#undef  CONFIG_SCSI_CONSTANTS
#undef  CONFIG_SCSI_LOGGING

/*
 * SCSI low-level drivers
 */
#undef  CONFIG_BLK_DEV_3W_XXXX_RAID
#define CONFIG_BLK_DEV_3W_XXXX_RAID_MODULE 1
#undef  CONFIG_SCSI_7000FASST
#define CONFIG_SCSI_7000FASST_MODULE 1
#undef  CONFIG_SCSI_ACARD
#define CONFIG_SCSI_ACARD_MODULE 1
#undef  CONFIG_SCSI_AHA152X
#define CONFIG_SCSI_AHA152X_MODULE 1
#undef  CONFIG_SCSI_AHA1542
#define CONFIG_SCSI_AHA1542_MODULE 1
#undef  CONFIG_SCSI_AHA1740
#define CONFIG_SCSI_AHA1740_MODULE 1
#undef  CONFIG_SCSI_AACRAID
#define CONFIG_SCSI_AACRAID_MODULE 1
#undef  CONFIG_SCSI_AIC7XXX
#define CONFIG_SCSI_AIC7XXX_MODULE 1
#define CONFIG_AIC7XXX_CMDS_PER_DEVICE (8)
#define CONFIG_AIC7XXX_RESET_DELAY_MS (15000)
#undef  CONFIG_AIC7XXX_PROBE_EISA_VL
#undef  CONFIG_AIC7XXX_BUILD_FIRMWARE
#undef  CONFIG_AIC7XXX_DEBUG_ENABLE
#define CONFIG_AIC7XXX_DEBUG_MASK (0)
#undef  CONFIG_AIC7XXX_REG_PRETTY_PRINT
#undef  CONFIG_SCSI_AIC79XX
#define CONFIG_SCSI_AIC79XX_MODULE 1
#define CONFIG_AIC79XX_CMDS_PER_DEVICE (32)
#define CONFIG_AIC79XX_RESET_DELAY_MS (15000)
#undef  CONFIG_AIC79XX_BUILD_FIRMWARE
#undef  CONFIG_AIC79XX_ENABLE_RD_STRM
#undef  CONFIG_AIC79XX_DEBUG_ENABLE
#define CONFIG_AIC79XX_DEBUG_MASK (0)
#undef  CONFIG_AIC79XX_REG_PRETTY_PRINT
#undef  CONFIG_SCSI_AIC7XXX_OLD
#define CONFIG_SCSI_AIC7XXX_OLD_MODULE 1
#undef  CONFIG_AIC7XXX_OLD_TCQ_ON_BY_DEFAULT
#define CONFIG_AIC7XXX_OLD_CMDS_PER_DEVICE (8)
#undef  CONFIG_AIC7XXX_OLD_PROC_STATS
#undef  CONFIG_SCSI_DPT_I2O
#define CONFIG_SCSI_DPT_I2O_MODULE 1
#undef  CONFIG_SCSI_ADVANSYS
#define CONFIG_SCSI_ADVANSYS_MODULE 1
#undef  CONFIG_SCSI_IN2000
#define CONFIG_SCSI_IN2000_MODULE 1
#undef  CONFIG_SCSI_AM53C974
#define CONFIG_SCSI_AM53C974_MODULE 1
#undef  CONFIG_SCSI_MEGARAID
#define CONFIG_SCSI_MEGARAID_MODULE 1
#undef  CONFIG_SCSI_MEGARAID2
#define CONFIG_SCSI_MEGARAID2_MODULE 1
#undef  CONFIG_SCSI_BUSLOGIC
#define CONFIG_SCSI_BUSLOGIC_MODULE 1
#undef  CONFIG_SCSI_OMIT_FLASHPOINT
#undef  CONFIG_SCSI_CPQFCTS
#define CONFIG_SCSI_CPQFCTS_MODULE 1
#undef  CONFIG_SCSI_DMX3191D
#define CONFIG_SCSI_DMX3191D_MODULE 1
#undef  CONFIG_SCSI_DTC3280
#define CONFIG_SCSI_DTC3280_MODULE 1
#undef  CONFIG_SCSI_EATA
#define CONFIG_SCSI_EATA_MODULE 1
#undef  CONFIG_SCSI_EATA_TAGGED_QUEUE
#undef  CONFIG_SCSI_EATA_LINKED_COMMANDS
#define CONFIG_SCSI_EATA_MAX_TAGS (16)
#undef  CONFIG_SCSI_EATA_DMA
#define CONFIG_SCSI_EATA_DMA_MODULE 1
#undef  CONFIG_SCSI_EATA_PIO
#define CONFIG_SCSI_EATA_PIO_MODULE 1
#undef  CONFIG_SCSI_FUTURE_DOMAIN
#define CONFIG_SCSI_FUTURE_DOMAIN_MODULE 1
#undef  CONFIG_SCSI_GDTH
#define CONFIG_SCSI_GDTH_MODULE 1
#undef  CONFIG_SCSI_GENERIC_NCR5380
#define CONFIG_SCSI_GENERIC_NCR5380_MODULE 1
#define CONFIG_SCSI_GENERIC_NCR53C400 1
#define CONFIG_SCSI_G_NCR5380_PORT 1
#undef  CONFIG_SCSI_G_NCR5380_MEM
#undef  CONFIG_SCSI_IPS
#define CONFIG_SCSI_IPS_MODULE 1
#undef  CONFIG_SCSI_INITIO
#define CONFIG_SCSI_INITIO_MODULE 1
#undef  CONFIG_SCSI_INIA100
#define CONFIG_SCSI_INIA100_MODULE 1
#undef  CONFIG_SCSI_PPA
#define CONFIG_SCSI_PPA_MODULE 1
#undef  CONFIG_SCSI_IMM
#define CONFIG_SCSI_IMM_MODULE 1
#define CONFIG_SCSI_IZIP_EPP16 1
#undef  CONFIG_SCSI_IZIP_SLOW_CTR
#undef  CONFIG_SCSI_NCR53C406A
#define CONFIG_SCSI_NCR53C406A_MODULE 1
#undef  CONFIG_SCSI_NCR53C7xx
#define CONFIG_SCSI_NCR53C7xx_MODULE 1
#undef  CONFIG_SCSI_NCR53C7xx_sync
#undef  CONFIG_SCSI_NCR53C7xx_FAST
#undef  CONFIG_SCSI_NCR53C7xx_DISCONNECT
#undef  CONFIG_SCSI_SYM53C8XX_2
#define CONFIG_SCSI_SYM53C8XX_2_MODULE 1
#define CONFIG_SCSI_SYM53C8XX_DMA_ADDRESSING_MODE (1)
#define CONFIG_SCSI_SYM53C8XX_DEFAULT_TAGS (16)
#define CONFIG_SCSI_SYM53C8XX_MAX_TAGS (64)
#undef  CONFIG_SCSI_SYM53C8XX_IOMAPPED
#undef  CONFIG_SCSI_NCR53C8XX
#define CONFIG_SCSI_NCR53C8XX_MODULE 1
#undef  CONFIG_SCSI_SYM53C8XX
#define CONFIG_SCSI_SYM53C8XX_MODULE 1
#define CONFIG_SCSI_NCR53C8XX_DEFAULT_TAGS (8)
#define CONFIG_SCSI_NCR53C8XX_MAX_TAGS (32)
#define CONFIG_SCSI_NCR53C8XX_SYNC (20)
#undef  CONFIG_SCSI_NCR53C8XX_PROFILE
#undef  CONFIG_SCSI_NCR53C8XX_IOMAPPED
#define CONFIG_SCSI_NCR53C8XX_PQS_PDS 1
#undef  CONFIG_SCSI_NCR53C8XX_SYMBIOS_COMPAT
#undef  CONFIG_SCSI_PAS16
#define CONFIG_SCSI_PAS16_MODULE 1
#undef  CONFIG_SCSI_PCI2000
#define CONFIG_SCSI_PCI2000_MODULE 1
#undef  CONFIG_SCSI_PCI2220I
#define CONFIG_SCSI_PCI2220I_MODULE 1
#undef  CONFIG_SCSI_PSI240I
#define CONFIG_SCSI_PSI240I_MODULE 1
#undef  CONFIG_SCSI_QLOGIC_FAS
#define CONFIG_SCSI_QLOGIC_FAS_MODULE 1
#undef  CONFIG_SCSI_QLOGIC_ISP
#define CONFIG_SCSI_QLOGIC_ISP_MODULE 1
#undef  CONFIG_SCSI_QLOGIC_FC
#define CONFIG_SCSI_QLOGIC_FC_MODULE 1
#define CONFIG_SCSI_QLOGIC_FC_FIRMWARE 1
#undef  CONFIG_SCSI_QLOGIC_1280
#define CONFIG_SCSI_QLOGIC_1280_MODULE 1
#undef  CONFIG_SCSI_SEAGATE
#define CONFIG_SCSI_SEAGATE_MODULE 1
#undef  CONFIG_SCSI_SIM710
#define CONFIG_SCSI_SIM710_MODULE 1
#undef  CONFIG_SCSI_SYM53C416
#define CONFIG_SCSI_SYM53C416_MODULE 1
#undef  CONFIG_SCSI_DC390T
#define CONFIG_SCSI_DC390T_MODULE 1
#undef  CONFIG_SCSI_DC390T_NOGENSUPP
#undef  CONFIG_SCSI_T128
#define CONFIG_SCSI_T128_MODULE 1
#undef  CONFIG_SCSI_U14_34F
#define CONFIG_SCSI_U14_34F_MODULE 1
#undef  CONFIG_SCSI_U14_34F_LINKED_COMMANDS
#define CONFIG_SCSI_U14_34F_MAX_TAGS (8)
#undef  CONFIG_SCSI_ULTRASTOR
#define CONFIG_SCSI_ULTRASTOR_MODULE 1
#undef  CONFIG_SCSI_NSP32
#define CONFIG_SCSI_NSP32_MODULE 1
#undef  CONFIG_SCSI_DEBUG
#define CONFIG_SCSI_DEBUG_MODULE 1

/*
 * PCMCIA SCSI adapter support
 */
#define CONFIG_SCSI_PCMCIA 1
#undef  CONFIG_PCMCIA_AHA152X
#define CONFIG_PCMCIA_AHA152X_MODULE 1
#undef  CONFIG_PCMCIA_FDOMAIN
#define CONFIG_PCMCIA_FDOMAIN_MODULE 1
#undef  CONFIG_PCMCIA_NINJA_SCSI
#define CONFIG_PCMCIA_NINJA_SCSI_MODULE 1
#undef  CONFIG_PCMCIA_QLOGIC
#define CONFIG_PCMCIA_QLOGIC_MODULE 1

/*
 * Fusion MPT device support
 */
#undef  CONFIG_FUSION
#define CONFIG_FUSION_MODULE 1
#undef  CONFIG_FUSION_BOOT
#define CONFIG_FUSION_MAX_SGE (40)
#undef  CONFIG_FUSION_ISENSE
#define CONFIG_FUSION_ISENSE_MODULE 1
#undef  CONFIG_FUSION_CTL
#define CONFIG_FUSION_CTL_MODULE 1
#undef  CONFIG_FUSION_LAN
#define CONFIG_FUSION_LAN_MODULE 1
#define CONFIG_NET_FC 1

/*
 * IEEE 1394 (FireWire) support (EXPERIMENTAL)
 */
#undef  CONFIG_IEEE1394
#define CONFIG_IEEE1394_MODULE 1

/*
 * Device Drivers
 */
#undef  CONFIG_IEEE1394_PCILYNX
#define CONFIG_IEEE1394_PCILYNX_MODULE 1
#undef  CONFIG_IEEE1394_OHCI1394
#define CONFIG_IEEE1394_OHCI1394_MODULE 1

/*
 * Protocol Drivers
 */
#undef  CONFIG_IEEE1394_VIDEO1394
#define CONFIG_IEEE1394_VIDEO1394_MODULE 1
#undef  CONFIG_IEEE1394_SBP2
#define CONFIG_IEEE1394_SBP2_MODULE 1
#undef  CONFIG_IEEE1394_SBP2_PHYS_DMA
#undef  CONFIG_IEEE1394_ETH1394
#define CONFIG_IEEE1394_ETH1394_MODULE 1
#undef  CONFIG_IEEE1394_DV1394
#define CONFIG_IEEE1394_DV1394_MODULE 1
#undef  CONFIG_IEEE1394_RAWIO
#define CONFIG_IEEE1394_RAWIO_MODULE 1
#undef  CONFIG_IEEE1394_CMP
#define CONFIG_IEEE1394_CMP_MODULE 1
#undef  CONFIG_IEEE1394_AMDTP
#define CONFIG_IEEE1394_AMDTP_MODULE 1
#undef  CONFIG_IEEE1394_VERBOSEDEBUG
#undef  CONFIG_IEEE1394_OUI_DB

/*
 * I2O device support
 */
#undef  CONFIG_I2O
#define CONFIG_I2O_MODULE 1
#undef  CONFIG_I2O_PCI
#define CONFIG_I2O_PCI_MODULE 1
#undef  CONFIG_I2O_BLOCK
#define CONFIG_I2O_BLOCK_MODULE 1
#undef  CONFIG_I2O_LAN
#define CONFIG_I2O_LAN_MODULE 1
#undef  CONFIG_I2O_SCSI
#define CONFIG_I2O_SCSI_MODULE 1
#undef  CONFIG_I2O_PROC
#define CONFIG_I2O_PROC_MODULE 1

/*
 * Network device support
 */
#define CONFIG_NETDEVICES 1

/*
 * ARCnet devices
 */
#undef  CONFIG_ARCNET
#define CONFIG_ARCNET_MODULE 1
#undef  CONFIG_ARCNET_1201
#define CONFIG_ARCNET_1201_MODULE 1
#undef  CONFIG_ARCNET_1051
#define CONFIG_ARCNET_1051_MODULE 1
#undef  CONFIG_ARCNET_RAW
#define CONFIG_ARCNET_RAW_MODULE 1
#undef  CONFIG_ARCNET_COM90xx
#define CONFIG_ARCNET_COM90xx_MODULE 1
#undef  CONFIG_ARCNET_COM90xxIO
#define CONFIG_ARCNET_COM90xxIO_MODULE 1
#undef  CONFIG_ARCNET_RIM_I
#define CONFIG_ARCNET_RIM_I_MODULE 1
#undef  CONFIG_ARCNET_COM20020
#define CONFIG_ARCNET_COM20020_MODULE 1
#undef  CONFIG_ARCNET_COM20020_ISA
#define CONFIG_ARCNET_COM20020_ISA_MODULE 1
#undef  CONFIG_ARCNET_COM20020_PCI
#define CONFIG_ARCNET_COM20020_PCI_MODULE 1
#undef  CONFIG_DUMMY
#define CONFIG_DUMMY_MODULE 1
#undef  CONFIG_BONDING
#define CONFIG_BONDING_MODULE 1
#undef  CONFIG_EQUALIZER
#define CONFIG_EQUALIZER_MODULE 1
#undef  CONFIG_TUN
#define CONFIG_TUN_MODULE 1
#undef  CONFIG_ETHERTAP
#define CONFIG_ETHERTAP_MODULE 1
#undef  CONFIG_NET_SB1000
#define CONFIG_NET_SB1000_MODULE 1

/*
 * Ethernet (10 or 100Mbit)
 */
#define CONFIG_NET_ETHERNET 1
#undef  CONFIG_SUNLANCE
#undef  CONFIG_HAPPYMEAL
#define CONFIG_HAPPYMEAL_MODULE 1
#undef  CONFIG_SUNBMAC
#undef  CONFIG_SUNQE
#undef  CONFIG_SUNGEM
#define CONFIG_SUNGEM_MODULE 1
#define CONFIG_NET_VENDOR_3COM 1
#undef  CONFIG_EL1
#define CONFIG_EL1_MODULE 1
#undef  CONFIG_EL2
#define CONFIG_EL2_MODULE 1
#undef  CONFIG_ELPLUS
#define CONFIG_ELPLUS_MODULE 1
#undef  CONFIG_EL16
#define CONFIG_EL16_MODULE 1
#undef  CONFIG_EL3
#define CONFIG_EL3_MODULE 1
#undef  CONFIG_3C515
#define CONFIG_3C515_MODULE 1
#undef  CONFIG_ELMC
#undef  CONFIG_ELMC_II
#undef  CONFIG_VORTEX
#define CONFIG_VORTEX_MODULE 1
#undef  CONFIG_TYPHOON
#define CONFIG_TYPHOON_MODULE 1
#undef  CONFIG_LANCE
#define CONFIG_LANCE_MODULE 1
#define CONFIG_NET_VENDOR_SMC 1
#undef  CONFIG_WD80x3
#define CONFIG_WD80x3_MODULE 1
#undef  CONFIG_ULTRAMCA
#undef  CONFIG_ULTRA
#define CONFIG_ULTRA_MODULE 1
#undef  CONFIG_ULTRA32
#undef  CONFIG_SMC9194
#define CONFIG_SMC9194_MODULE 1
#define CONFIG_NET_VENDOR_RACAL 1
#undef  CONFIG_NI5010
#define CONFIG_NI5010_MODULE 1
#undef  CONFIG_NI52
#define CONFIG_NI52_MODULE 1
#undef  CONFIG_NI65
#define CONFIG_NI65_MODULE 1
#undef  CONFIG_AT1700
#define CONFIG_AT1700_MODULE 1
#undef  CONFIG_DEPCA
#define CONFIG_DEPCA_MODULE 1
#undef  CONFIG_HP100
#define CONFIG_HP100_MODULE 1
#define CONFIG_NET_ISA 1
#undef  CONFIG_E2100
#define CONFIG_E2100_MODULE 1
#undef  CONFIG_EWRK3
#define CONFIG_EWRK3_MODULE 1
#undef  CONFIG_EEXPRESS
#define CONFIG_EEXPRESS_MODULE 1
#undef  CONFIG_EEXPRESS_PRO
#define CONFIG_EEXPRESS_PRO_MODULE 1
#undef  CONFIG_HPLAN_PLUS
#define CONFIG_HPLAN_PLUS_MODULE 1
#undef  CONFIG_HPLAN
#define CONFIG_HPLAN_MODULE 1
#undef  CONFIG_LP486E
#define CONFIG_LP486E_MODULE 1
#undef  CONFIG_ETH16I
#define CONFIG_ETH16I_MODULE 1
#undef  CONFIG_NE2000
#define CONFIG_NE2000_MODULE 1
#define CONFIG_NET_PCI 1
#undef  CONFIG_PCNET32
#define CONFIG_PCNET32_MODULE 1
#undef  CONFIG_AMD8111_ETH
#define CONFIG_AMD8111_ETH_MODULE 1
#undef  CONFIG_ADAPTEC_STARFIRE
#define CONFIG_ADAPTEC_STARFIRE_MODULE 1
#undef  CONFIG_AC3200
#define CONFIG_AC3200_MODULE 1
#undef  CONFIG_APRICOT
#define CONFIG_APRICOT_MODULE 1
#undef  CONFIG_B44
#define CONFIG_B44_MODULE 1
#undef  CONFIG_CS89x0
#define CONFIG_CS89x0_MODULE 1
#undef  CONFIG_TULIP
#define CONFIG_TULIP_MODULE 1
#undef  CONFIG_TULIP_MWI
#undef  CONFIG_TULIP_MMIO
#undef  CONFIG_DE4X5
#define CONFIG_DE4X5_MODULE 1
#undef  CONFIG_DGRS
#define CONFIG_DGRS_MODULE 1
#undef  CONFIG_DM9102
#define CONFIG_DM9102_MODULE 1
#undef  CONFIG_EEPRO100
#define CONFIG_EEPRO100_MODULE 1
#undef  CONFIG_EEPRO100_PIO
#undef  CONFIG_E100
#define CONFIG_E100_MODULE 1
#undef  CONFIG_LNE390
#undef  CONFIG_FEALNX
#define CONFIG_FEALNX_MODULE 1
#undef  CONFIG_NATSEMI
#define CONFIG_NATSEMI_MODULE 1
#undef  CONFIG_NE2K_PCI
#define CONFIG_NE2K_PCI_MODULE 1
#undef  CONFIG_FORCEDETH
#define CONFIG_FORCEDETH_MODULE 1
#undef  CONFIG_NE3210
#undef  CONFIG_ES3210
#undef  CONFIG_8139CP
#define CONFIG_8139CP_MODULE 1
#undef  CONFIG_8139TOO
#define CONFIG_8139TOO_MODULE 1
#undef  CONFIG_8139TOO_PIO
#define CONFIG_8139TOO_TUNE_TWISTER 1
#define CONFIG_8139TOO_8129 1
#undef  CONFIG_8139_OLD_RX_RESET
#undef  CONFIG_SIS900
#define CONFIG_SIS900_MODULE 1
#undef  CONFIG_EPIC100
#define CONFIG_EPIC100_MODULE 1
#undef  CONFIG_SUNDANCE
#define CONFIG_SUNDANCE_MODULE 1
#undef  CONFIG_SUNDANCE_MMIO
#undef  CONFIG_TLAN
#define CONFIG_TLAN_MODULE 1
#undef  CONFIG_VIA_RHINE
#define CONFIG_VIA_RHINE_MODULE 1
#undef  CONFIG_VIA_RHINE_MMIO
#undef  CONFIG_WINBOND_840
#define CONFIG_WINBOND_840_MODULE 1
#define CONFIG_NET_POCKET 1
#undef  CONFIG_ATP
#define CONFIG_ATP_MODULE 1
#undef  CONFIG_DE600
#define CONFIG_DE600_MODULE 1
#undef  CONFIG_DE620
#define CONFIG_DE620_MODULE 1

/*
 * Ethernet (1000 Mbit)
 */
#undef  CONFIG_ACENIC
#define CONFIG_ACENIC_MODULE 1
#undef  CONFIG_ACENIC_OMIT_TIGON_I
#undef  CONFIG_DL2K
#define CONFIG_DL2K_MODULE 1
#undef  CONFIG_E1000
#define CONFIG_E1000_MODULE 1
#undef  CONFIG_E1000_NAPI
#undef  CONFIG_MYRI_SBUS
#undef  CONFIG_NS83820
#define CONFIG_NS83820_MODULE 1
#undef  CONFIG_HAMACHI
#define CONFIG_HAMACHI_MODULE 1
#undef  CONFIG_YELLOWFIN
#define CONFIG_YELLOWFIN_MODULE 1
#undef  CONFIG_R8169
#define CONFIG_R8169_MODULE 1
#undef  CONFIG_SK98LIN
#define CONFIG_SK98LIN_MODULE 1
#undef  CONFIG_TIGON3
#define CONFIG_TIGON3_MODULE 1
#define CONFIG_FDDI 1
#undef  CONFIG_DEFXX
#define CONFIG_DEFXX_MODULE 1
#undef  CONFIG_SKFP
#define CONFIG_SKFP_MODULE 1
#undef  CONFIG_HIPPI
#undef  CONFIG_PLIP
#define CONFIG_PLIP_MODULE 1
#undef  CONFIG_PPP
#define CONFIG_PPP_MODULE 1
#define CONFIG_PPP_MULTILINK 1
#define CONFIG_PPP_FILTER 1
#undef  CONFIG_PPP_ASYNC
#define CONFIG_PPP_ASYNC_MODULE 1
#undef  CONFIG_PPP_SYNC_TTY
#define CONFIG_PPP_SYNC_TTY_MODULE 1
#undef  CONFIG_PPP_DEFLATE
#define CONFIG_PPP_DEFLATE_MODULE 1
#undef  CONFIG_PPP_BSDCOMP
#define CONFIG_PPP_BSDCOMP_MODULE 1
#undef  CONFIG_PPPOE
#define CONFIG_PPPOE_MODULE 1
#undef  CONFIG_SLIP
#define CONFIG_SLIP_MODULE 1
#define CONFIG_SLIP_COMPRESSED 1
#define CONFIG_SLIP_SMART 1
#undef  CONFIG_SLIP_MODE_SLIP6

/*
 * Wireless LAN (non-hamradio)
 */
#define CONFIG_NET_RADIO 1
#undef  CONFIG_STRIP
#define CONFIG_STRIP_MODULE 1
#undef  CONFIG_WAVELAN
#define CONFIG_WAVELAN_MODULE 1
#undef  CONFIG_ARLAN
#define CONFIG_ARLAN_MODULE 1
#undef  CONFIG_AIRONET4500
#define CONFIG_AIRONET4500_MODULE 1
#undef  CONFIG_AIRONET4500_NONCS
#define CONFIG_AIRONET4500_NONCS_MODULE 1
#define CONFIG_AIRONET4500_PNP 1
#define CONFIG_AIRONET4500_PCI 1
#undef  CONFIG_AIRONET4500_ISA
#undef  CONFIG_AIRONET4500_I365
#undef  CONFIG_AIRONET4500_PROC
#define CONFIG_AIRONET4500_PROC_MODULE 1
#undef  CONFIG_AIRO
#define CONFIG_AIRO_MODULE 1
#undef  CONFIG_HERMES
#define CONFIG_HERMES_MODULE 1
#undef  CONFIG_PLX_HERMES
#define CONFIG_PLX_HERMES_MODULE 1
#undef  CONFIG_TMD_HERMES
#define CONFIG_TMD_HERMES_MODULE 1
#undef  CONFIG_PCI_HERMES
#define CONFIG_PCI_HERMES_MODULE 1

/*
 * Wireless Pcmcia cards support
 */
#undef  CONFIG_PCMCIA_HERMES
#define CONFIG_PCMCIA_HERMES_MODULE 1
#undef  CONFIG_AIRO_CS
#define CONFIG_AIRO_CS_MODULE 1
#undef  CONFIG_PCMCIA_ATMEL
#define CONFIG_PCMCIA_ATMEL_MODULE 1
#define CONFIG_NET_WIRELESS 1

/*
 * Token Ring devices
 */
#define CONFIG_TR 1
#undef  CONFIG_IBMTR
#define CONFIG_IBMTR_MODULE 1
#undef  CONFIG_IBMOL
#define CONFIG_IBMOL_MODULE 1
#undef  CONFIG_IBMLS
#define CONFIG_IBMLS_MODULE 1
#undef  CONFIG_3C359
#define CONFIG_3C359_MODULE 1
#undef  CONFIG_TMS380TR
#define CONFIG_TMS380TR_MODULE 1
#undef  CONFIG_TMSPCI
#define CONFIG_TMSPCI_MODULE 1
#undef  CONFIG_TMSISA
#define CONFIG_TMSISA_MODULE 1
#undef  CONFIG_ABYSS
#define CONFIG_ABYSS_MODULE 1
#undef  CONFIG_MADGEMC
#undef  CONFIG_SMCTR
#define CONFIG_SMCTR_MODULE 1
#define CONFIG_NET_FC 1
#undef  CONFIG_IPHASE5526
#define CONFIG_IPHASE5526_MODULE 1
#undef  CONFIG_RCPCI
#define CONFIG_RCPCI_MODULE 1
#undef  CONFIG_SHAPER
#define CONFIG_SHAPER_MODULE 1

/*
 * Wan interfaces
 */
#define CONFIG_WAN 1
#undef  CONFIG_HOSTESS_SV11
#define CONFIG_HOSTESS_SV11_MODULE 1
#undef  CONFIG_COSA
#define CONFIG_COSA_MODULE 1
#undef  CONFIG_COMX
#undef  CONFIG_DSCC4
#define CONFIG_DSCC4_MODULE 1
#undef  CONFIG_DSCC4_PCISYNC
#undef  CONFIG_DSCC4_PCI_RST
#undef  CONFIG_LANMEDIA
#define CONFIG_LANMEDIA_MODULE 1
#undef  CONFIG_ATI_XX20
#define CONFIG_ATI_XX20_MODULE 1
#undef  CONFIG_SEALEVEL_4021
#define CONFIG_SEALEVEL_4021_MODULE 1
#undef  CONFIG_SYNCLINK_SYNCPPP
#define CONFIG_SYNCLINK_SYNCPPP_MODULE 1
#undef  CONFIG_HDLC
#define CONFIG_HDLC_MODULE 1
#define CONFIG_HDLC_RAW 1
#define CONFIG_HDLC_RAW_ETH 1
#define CONFIG_HDLC_CISCO 1
#define CONFIG_HDLC_FR 1
#undef  CONFIG_HDLC_PPP
#undef  CONFIG_HDLC_X25
#undef  CONFIG_PCI200SYN
#define CONFIG_PCI200SYN_MODULE 1
#undef  CONFIG_PC300
#define CONFIG_PC300_MODULE 1

/*
 *      Cyclades-PC300 MLPPP support is disabled.
 */

/*
 *      Refer to the file README.mlppp, provided by PC300 package.
 */
#undef  CONFIG_FARSYNC
#define CONFIG_FARSYNC_MODULE 1
#undef  CONFIG_N2
#define CONFIG_N2_MODULE 1
#undef  CONFIG_C101
#define CONFIG_C101_MODULE 1
#undef  CONFIG_DLCI
#define CONFIG_DLCI_MODULE 1
#define CONFIG_DLCI_COUNT (24)
#define CONFIG_DLCI_MAX (8)
#undef  CONFIG_SDLA
#define CONFIG_SDLA_MODULE 1
#define CONFIG_WAN_ROUTER_DRIVERS 1
#undef  CONFIG_VENDOR_SANGOMA
#define CONFIG_VENDOR_SANGOMA_MODULE 1
#define CONFIG_WANPIPE_CHDLC 1
#undef  CONFIG_WANPIPE_FR
#undef  CONFIG_WANPIPE_X25
#define CONFIG_WANPIPE_PPP 1
#undef  CONFIG_WANPIPE_MULTPPP
#undef  CONFIG_CYCLADES_SYNC
#define CONFIG_CYCLADES_SYNC_MODULE 1
#define CONFIG_CYCLOMX_X25 1
#undef  CONFIG_LAPBETHER
#define CONFIG_LAPBETHER_MODULE 1
#undef  CONFIG_X25_ASY
#define CONFIG_X25_ASY_MODULE 1
#undef  CONFIG_SBNI
#define CONFIG_SBNI_MODULE 1
#undef  CONFIG_SBNI_MULTILINE

/*
 * PCMCIA network device support
 */
#define CONFIG_NET_PCMCIA 1
#undef  CONFIG_PCMCIA_3C589
#define CONFIG_PCMCIA_3C589_MODULE 1
#undef  CONFIG_PCMCIA_3C574
#define CONFIG_PCMCIA_3C574_MODULE 1
#undef  CONFIG_PCMCIA_FMVJ18X
#define CONFIG_PCMCIA_FMVJ18X_MODULE 1
#undef  CONFIG_PCMCIA_PCNET
#define CONFIG_PCMCIA_PCNET_MODULE 1
#undef  CONFIG_PCMCIA_AXNET
#define CONFIG_PCMCIA_AXNET_MODULE 1
#undef  CONFIG_PCMCIA_NMCLAN
#define CONFIG_PCMCIA_NMCLAN_MODULE 1
#undef  CONFIG_PCMCIA_SMC91C92
#define CONFIG_PCMCIA_SMC91C92_MODULE 1
#undef  CONFIG_PCMCIA_XIRC2PS
#define CONFIG_PCMCIA_XIRC2PS_MODULE 1
#undef  CONFIG_ARCNET_COM20020_CS
#define CONFIG_ARCNET_COM20020_CS_MODULE 1
#undef  CONFIG_PCMCIA_IBMTR
#define CONFIG_PCMCIA_IBMTR_MODULE 1
#undef  CONFIG_PCMCIA_XIRCOM
#define CONFIG_PCMCIA_XIRCOM_MODULE 1
#undef  CONFIG_PCMCIA_XIRTULIP
#define CONFIG_PCMCIA_XIRTULIP_MODULE 1
#define CONFIG_NET_PCMCIA_RADIO 1
#undef  CONFIG_PCMCIA_RAYCS
#define CONFIG_PCMCIA_RAYCS_MODULE 1
#undef  CONFIG_PCMCIA_NETWAVE
#define CONFIG_PCMCIA_NETWAVE_MODULE 1
#undef  CONFIG_PCMCIA_WAVELAN
#define CONFIG_PCMCIA_WAVELAN_MODULE 1
#undef  CONFIG_AIRONET4500_CS
#define CONFIG_AIRONET4500_CS_MODULE 1

/*
 * Amateur Radio support
 */
#define CONFIG_HAMRADIO 1

/*
 * Packet Radio protocols
 */
#undef  CONFIG_AX25
#define CONFIG_AX25_MODULE 1
#define CONFIG_AX25_DAMA_SLAVE 1
#undef  CONFIG_NETROM
#define CONFIG_NETROM_MODULE 1
#undef  CONFIG_ROSE
#define CONFIG_ROSE_MODULE 1

/*
 * AX.25 network device drivers
 */

/*
 * AX.25 network device drivers
 */
#undef  CONFIG_MKISS
#define CONFIG_MKISS_MODULE 1
#undef  CONFIG_6PACK
#define CONFIG_6PACK_MODULE 1
#undef  CONFIG_BPQETHER
#define CONFIG_BPQETHER_MODULE 1
#undef  CONFIG_DMASCC
#define CONFIG_DMASCC_MODULE 1
#undef  CONFIG_SCC
#define CONFIG_SCC_MODULE 1
#define CONFIG_SCC_DELAY 1
#define CONFIG_SCC_TRXECHO 1
#undef  CONFIG_BAYCOM_SER_FDX
#define CONFIG_BAYCOM_SER_FDX_MODULE 1
#undef  CONFIG_BAYCOM_SER_HDX
#define CONFIG_BAYCOM_SER_HDX_MODULE 1
#undef  CONFIG_BAYCOM_PAR
#define CONFIG_BAYCOM_PAR_MODULE 1
#undef  CONFIG_BAYCOM_EPP
#define CONFIG_BAYCOM_EPP_MODULE 1
#undef  CONFIG_SOUNDMODEM
#define CONFIG_SOUNDMODEM_MODULE 1
#define CONFIG_SOUNDMODEM_SBC 1
#define CONFIG_SOUNDMODEM_WSS 1
#define CONFIG_SOUNDMODEM_AFSK1200 1
#define CONFIG_SOUNDMODEM_AFSK2400_7 1
#define CONFIG_SOUNDMODEM_AFSK2400_8 1
#define CONFIG_SOUNDMODEM_AFSK2666 1
#define CONFIG_SOUNDMODEM_HAPN4800 1
#define CONFIG_SOUNDMODEM_PSK4800 1
#define CONFIG_SOUNDMODEM_FSK9600 1
#undef  CONFIG_YAM
#define CONFIG_YAM_MODULE 1

/*
 * IrDA (infrared) support
 */
#undef  CONFIG_IRDA
#define CONFIG_IRDA_MODULE 1

/*
 * IrDA protocols
 */
#undef  CONFIG_IRLAN
#define CONFIG_IRLAN_MODULE 1
#undef  CONFIG_IRNET
#define CONFIG_IRNET_MODULE 1
#undef  CONFIG_IRCOMM
#define CONFIG_IRCOMM_MODULE 1
#undef  CONFIG_IRDA_ULTRA

/*
 * IrDA options
 */
#define CONFIG_IRDA_CACHE_LAST_LSAP 1
#define CONFIG_IRDA_FAST_RR 1
#undef  CONFIG_IRDA_DEBUG

/*
 * Infrared-port device drivers
 */

/*
 * SIR device drivers
 */
#undef  CONFIG_IRTTY_SIR
#define CONFIG_IRTTY_SIR_MODULE 1
#undef  CONFIG_IRPORT_SIR
#define CONFIG_IRPORT_SIR_MODULE 1

/*
 * Dongle support
 */
#define CONFIG_DONGLE 1
#undef  CONFIG_ESI_DONGLE
#define CONFIG_ESI_DONGLE_MODULE 1
#undef  CONFIG_ACTISYS_DONGLE
#define CONFIG_ACTISYS_DONGLE_MODULE 1
#undef  CONFIG_TEKRAM_DONGLE
#define CONFIG_TEKRAM_DONGLE_MODULE 1
#undef  CONFIG_GIRBIL_DONGLE
#define CONFIG_GIRBIL_DONGLE_MODULE 1
#undef  CONFIG_LITELINK_DONGLE
#define CONFIG_LITELINK_DONGLE_MODULE 1
#undef  CONFIG_MCP2120_DONGLE
#define CONFIG_MCP2120_DONGLE_MODULE 1
#undef  CONFIG_OLD_BELKIN_DONGLE
#define CONFIG_OLD_BELKIN_DONGLE_MODULE 1
#undef  CONFIG_ACT200L_DONGLE
#define CONFIG_ACT200L_DONGLE_MODULE 1
#undef  CONFIG_MA600_DONGLE
#define CONFIG_MA600_DONGLE_MODULE 1

/*
 * FIR device drivers
 */
#undef  CONFIG_USB_IRDA
#define CONFIG_USB_IRDA_MODULE 1
#undef  CONFIG_NSC_FIR
#define CONFIG_NSC_FIR_MODULE 1
#undef  CONFIG_WINBOND_FIR
#define CONFIG_WINBOND_FIR_MODULE 1
#undef  CONFIG_TOSHIBA_OLD
#define CONFIG_TOSHIBA_OLD_MODULE 1
#undef  CONFIG_TOSHIBA_FIR
#define CONFIG_TOSHIBA_FIR_MODULE 1
#undef  CONFIG_SMC_IRCC_FIR
#define CONFIG_SMC_IRCC_FIR_MODULE 1
#undef  CONFIG_ALI_FIR
#define CONFIG_ALI_FIR_MODULE 1
#undef  CONFIG_VLSI_FIR
#define CONFIG_VLSI_FIR_MODULE 1
#undef  CONFIG_VIA_IRCC_FIR
#define CONFIG_VIA_IRCC_FIR_MODULE 1

/*
 * ISDN subsystem
 */
#undef  CONFIG_ISDN
#define CONFIG_ISDN_MODULE 1
#define CONFIG_ISDN_BOOL 1
#define CONFIG_ISDN_PPP 1
#define CONFIG_IPPP_FILTER 1
#define CONFIG_ISDN_PPP_VJ 1
#define CONFIG_ISDN_MPP 1
#undef  CONFIG_ISDN_PPP_BSDCOMP
#define CONFIG_ISDN_PPP_BSDCOMP_MODULE 1
#define CONFIG_ISDN_AUDIO 1
#undef  CONFIG_ISDN_TTY_FAX
#define CONFIG_ISDN_X25 1

/*
 * ISDN feature submodules
 */
#undef  CONFIG_ISDN_DRV_LOOP
#define CONFIG_ISDN_DRV_LOOP_MODULE 1
#undef  CONFIG_ISDN_DIVERSION
#define CONFIG_ISDN_DIVERSION_MODULE 1

/*
 * low-level hardware drivers
 */

/*
 * Passive ISDN cards
 */
#undef  CONFIG_ISDN_DRV_HISAX
#define CONFIG_ISDN_DRV_HISAX_MODULE 1
#define CONFIG_ISDN_HISAX 1

/*
 *   D-channel protocol features
 */
#define CONFIG_HISAX_EURO 1
#define CONFIG_DE_AOC 1
#undef  CONFIG_HISAX_NO_SENDCOMPLETE
#undef  CONFIG_HISAX_NO_LLC
#undef  CONFIG_HISAX_NO_KEYPAD
#define CONFIG_HISAX_1TR6 1
#undef  CONFIG_HISAX_NI1
#define CONFIG_HISAX_MAX_CARDS (8)

/*
 *   HiSax supported cards
 */
#define CONFIG_HISAX_16_0 1
#define CONFIG_HISAX_16_3 1
#define CONFIG_HISAX_AVM_A1 1
#define CONFIG_HISAX_IX1MICROR2 1
#define CONFIG_HISAX_ASUSCOM 1
#define CONFIG_HISAX_TELEINT 1
#define CONFIG_HISAX_HFCS 1
#define CONFIG_HISAX_SPORTSTER 1
#define CONFIG_HISAX_MIC 1
#define CONFIG_HISAX_ISURF 1
#define CONFIG_HISAX_HSTSAPHIR 1
#define CONFIG_HISAX_TELESPCI 1
#define CONFIG_HISAX_S0BOX 1
#define CONFIG_HISAX_FRITZPCI 1
#define CONFIG_HISAX_AVM_A1_PCMCIA 1
#define CONFIG_HISAX_ELSA 1
#define CONFIG_HISAX_DIEHLDIVA 1
#define CONFIG_HISAX_SEDLBAUER 1
#define CONFIG_HISAX_NETJET 1
#define CONFIG_HISAX_NETJET_U 1
#define CONFIG_HISAX_NICCY 1
#define CONFIG_HISAX_BKM_A4T 1
#define CONFIG_HISAX_SCT_QUADRO 1
#define CONFIG_HISAX_GAZEL 1
#define CONFIG_HISAX_HFC_PCI 1
#define CONFIG_HISAX_W6692 1
#define CONFIG_HISAX_HFC_SX 1
#define CONFIG_HISAX_ENTERNOW_PCI 1
#undef  CONFIG_HISAX_DEBUG
#undef  CONFIG_HISAX_SEDLBAUER_CS
#define CONFIG_HISAX_SEDLBAUER_CS_MODULE 1
#undef  CONFIG_HISAX_ELSA_CS
#define CONFIG_HISAX_ELSA_CS_MODULE 1
#undef  CONFIG_HISAX_AVM_A1_CS
#define CONFIG_HISAX_AVM_A1_CS_MODULE 1
#undef  CONFIG_HISAX_ST5481
#define CONFIG_HISAX_ST5481_MODULE 1
#undef  CONFIG_HISAX_FRITZ_PCIPNP
#define CONFIG_HISAX_FRITZ_PCIPNP_MODULE 1
#undef  CONFIG_USB_AUERISDN
#define CONFIG_USB_AUERISDN_MODULE 1

/*
 * Active ISDN cards
 */
#undef  CONFIG_ISDN_DRV_ICN
#define CONFIG_ISDN_DRV_ICN_MODULE 1
#undef  CONFIG_ISDN_DRV_PCBIT
#define CONFIG_ISDN_DRV_PCBIT_MODULE 1
#undef  CONFIG_ISDN_DRV_SC
#define CONFIG_ISDN_DRV_SC_MODULE 1
#undef  CONFIG_ISDN_DRV_ACT2000
#define CONFIG_ISDN_DRV_ACT2000_MODULE 1
#define CONFIG_ISDN_DRV_EICON 1
#undef  CONFIG_ISDN_DRV_EICON_DIVAS
#define CONFIG_ISDN_DRV_EICON_DIVAS_MODULE 1
#undef  CONFIG_ISDN_DRV_EICON_OLD
#define CONFIG_ISDN_DRV_EICON_OLD_MODULE 1
#define CONFIG_ISDN_DRV_EICON_PCI 1
#define CONFIG_ISDN_DRV_EICON_ISA 1
#undef  CONFIG_ISDN_DRV_TPAM
#define CONFIG_ISDN_DRV_TPAM_MODULE 1
#undef  CONFIG_ISDN_CAPI
#define CONFIG_ISDN_CAPI_MODULE 1
#define CONFIG_ISDN_DRV_AVMB1_VERBOSE_REASON 1
#undef  CONFIG_ISDN_CAPI_MIDDLEWARE
#undef  CONFIG_ISDN_CAPI_CAPI20
#define CONFIG_ISDN_CAPI_CAPI20_MODULE 1
#undef  CONFIG_ISDN_CAPI_CAPIDRV
#define CONFIG_ISDN_CAPI_CAPIDRV_MODULE 1
#undef  CONFIG_ISDN_DRV_AVMB1_B1ISA
#define CONFIG_ISDN_DRV_AVMB1_B1ISA_MODULE 1
#undef  CONFIG_ISDN_DRV_AVMB1_B1PCI
#define CONFIG_ISDN_DRV_AVMB1_B1PCI_MODULE 1
#define CONFIG_ISDN_DRV_AVMB1_B1PCIV4 1
#undef  CONFIG_ISDN_DRV_AVMB1_T1ISA
#define CONFIG_ISDN_DRV_AVMB1_T1ISA_MODULE 1
#undef  CONFIG_ISDN_DRV_AVMB1_B1PCMCIA
#define CONFIG_ISDN_DRV_AVMB1_B1PCMCIA_MODULE 1
#undef  CONFIG_ISDN_DRV_AVMB1_AVM_CS
#define CONFIG_ISDN_DRV_AVMB1_AVM_CS_MODULE 1
#undef  CONFIG_ISDN_DRV_AVMB1_T1PCI
#define CONFIG_ISDN_DRV_AVMB1_T1PCI_MODULE 1
#undef  CONFIG_ISDN_DRV_AVMB1_C4
#define CONFIG_ISDN_DRV_AVMB1_C4_MODULE 1
#undef  CONFIG_HYSDN
#define CONFIG_HYSDN_MODULE 1
#undef  CONFIG_HYSDN_CAPI

/*
 * Old CD-ROM drivers (not SCSI, not IDE)
 */
#define CONFIG_CD_NO_IDESCSI 1
#undef  CONFIG_AZTCD
#define CONFIG_AZTCD_MODULE 1
#undef  CONFIG_GSCD
#define CONFIG_GSCD_MODULE 1
#undef  CONFIG_SBPCD
#define CONFIG_SBPCD_MODULE 1
#undef  CONFIG_MCD
#define CONFIG_MCD_MODULE 1
#define CONFIG_MCD_IRQ (11)
#define CONFIG_MCD_BASE 0x300
#undef  CONFIG_MCDX
#define CONFIG_MCDX_MODULE 1
#undef  CONFIG_OPTCD
#define CONFIG_OPTCD_MODULE 1
#undef  CONFIG_CM206
#define CONFIG_CM206_MODULE 1
#undef  CONFIG_SJCD
#define CONFIG_SJCD_MODULE 1
#undef  CONFIG_ISP16_CDI
#define CONFIG_ISP16_CDI_MODULE 1
#undef  CONFIG_CDU31A
#define CONFIG_CDU31A_MODULE 1
#undef  CONFIG_CDU535
#define CONFIG_CDU535_MODULE 1

/*
 * Input core support
 */
#undef  CONFIG_INPUT
#define CONFIG_INPUT_MODULE 1
#undef  CONFIG_INPUT_KEYBDEV
#define CONFIG_INPUT_KEYBDEV_MODULE 1
#undef  CONFIG_INPUT_MOUSEDEV
#define CONFIG_INPUT_MOUSEDEV_MODULE 1
#define CONFIG_INPUT_MOUSEDEV_SCREEN_X (1024)
#define CONFIG_INPUT_MOUSEDEV_SCREEN_Y (768)
#undef  CONFIG_INPUT_JOYDEV
#define CONFIG_INPUT_JOYDEV_MODULE 1
#undef  CONFIG_INPUT_EVDEV
#define CONFIG_INPUT_EVDEV_MODULE 1
#undef  CONFIG_INPUT_UINPUT
#define CONFIG_INPUT_UINPUT_MODULE 1

/*
 * Character devices
 */
#define CONFIG_VT 1
#define CONFIG_VT_CONSOLE 1
#define CONFIG_SERIAL 1
#define CONFIG_SERIAL_CONSOLE 1
#define CONFIG_SERIAL_EXTENDED 1
#define CONFIG_SERIAL_MANY_PORTS 1
#define CONFIG_SERIAL_SHARE_IRQ 1
#undef  CONFIG_SERIAL_DETECT_IRQ
#define CONFIG_SERIAL_MULTIPORT 1
#define CONFIG_HUB6 1
#define CONFIG_SERIAL_NONSTANDARD 1
#undef  CONFIG_COMPUTONE
#define CONFIG_COMPUTONE_MODULE 1
#undef  CONFIG_ROCKETPORT
#define CONFIG_ROCKETPORT_MODULE 1
#undef  CONFIG_CYCLADES
#define CONFIG_CYCLADES_MODULE 1
#undef  CONFIG_CYZ_INTR
#undef  CONFIG_DIGIEPCA
#define CONFIG_DIGIEPCA_MODULE 1
#undef  CONFIG_ESPSERIAL
#define CONFIG_ESPSERIAL_MODULE 1
#undef  CONFIG_MOXA_INTELLIO
#define CONFIG_MOXA_INTELLIO_MODULE 1
#undef  CONFIG_MOXA_SMARTIO
#define CONFIG_MOXA_SMARTIO_MODULE 1
#undef  CONFIG_ISI
#define CONFIG_ISI_MODULE 1
#undef  CONFIG_SYNCLINK
#define CONFIG_SYNCLINK_MODULE 1
#undef  CONFIG_SYNCLINKMP
#define CONFIG_SYNCLINKMP_MODULE 1
#undef  CONFIG_N_HDLC
#define CONFIG_N_HDLC_MODULE 1
#undef  CONFIG_RISCOM8
#define CONFIG_RISCOM8_MODULE 1
#undef  CONFIG_SPECIALIX
#define CONFIG_SPECIALIX_MODULE 1
#undef  CONFIG_SPECIALIX_RTSCTS
#undef  CONFIG_SX
#define CONFIG_SX_MODULE 1
#undef  CONFIG_RIO
#define CONFIG_RIO_MODULE 1
#undef  CONFIG_RIO_OLDPCI
#define CONFIG_STALDRV 1
#undef  CONFIG_STALLION
#define CONFIG_STALLION_MODULE 1
#undef  CONFIG_ISTALLION
#define CONFIG_ISTALLION_MODULE 1
#define CONFIG_UNIX98_PTYS 1
#define CONFIG_UNIX98_PTY_COUNT (512)
#undef  CONFIG_PRINTER
#define CONFIG_PRINTER_MODULE 1
#undef  CONFIG_LP_CONSOLE
#undef  CONFIG_PPDEV
#define CONFIG_PPDEV_MODULE 1
#undef  CONFIG_TIPAR
#define CONFIG_TIPAR_MODULE 1

/*
 * I2C support
 */
#undef  CONFIG_I2C
#define CONFIG_I2C_MODULE 1
#undef  CONFIG_I2C_ALGOBIT
#define CONFIG_I2C_ALGOBIT_MODULE 1
#undef  CONFIG_I2C_PHILIPSPAR
#define CONFIG_I2C_PHILIPSPAR_MODULE 1
#undef  CONFIG_I2C_ELV
#define CONFIG_I2C_ELV_MODULE 1
#undef  CONFIG_I2C_VELLEMAN
#define CONFIG_I2C_VELLEMAN_MODULE 1
#undef  CONFIG_SCx200_I2C
#define CONFIG_SCx200_I2C_MODULE 1
#define CONFIG_SCx200_I2C_SCL (12)
#define CONFIG_SCx200_I2C_SDA (13)
#undef  CONFIG_SCx200_ACB
#define CONFIG_SCx200_ACB_MODULE 1
#undef  CONFIG_I2C_ALGOPCF
#define CONFIG_I2C_ALGOPCF_MODULE 1
#undef  CONFIG_I2C_ELEKTOR
#define CONFIG_I2C_ELEKTOR_MODULE 1
#undef  CONFIG_I2C_CHARDEV
#define CONFIG_I2C_CHARDEV_MODULE 1
#undef  CONFIG_I2C_PROC
#define CONFIG_I2C_PROC_MODULE 1

/*
 * Mice
 */
#undef  CONFIG_BUSMOUSE
#define CONFIG_BUSMOUSE_MODULE 1
#undef  CONFIG_ATIXL_BUSMOUSE
#define CONFIG_ATIXL_BUSMOUSE_MODULE 1
#undef  CONFIG_LOGIBUSMOUSE
#define CONFIG_LOGIBUSMOUSE_MODULE 1
#undef  CONFIG_MS_BUSMOUSE
#define CONFIG_MS_BUSMOUSE_MODULE 1
#define CONFIG_MOUSE 1
#define CONFIG_PSMOUSE 1
#undef  CONFIG_82C710_MOUSE
#define CONFIG_82C710_MOUSE_MODULE 1
#undef  CONFIG_PC110_PAD
#define CONFIG_PC110_PAD_MODULE 1
#undef  CONFIG_MK712_MOUSE
#define CONFIG_MK712_MOUSE_MODULE 1

/*
 * Joysticks
 */
#undef  CONFIG_INPUT_GAMEPORT
#define CONFIG_INPUT_GAMEPORT_MODULE 1
#undef  CONFIG_INPUT_NS558
#define CONFIG_INPUT_NS558_MODULE 1
#undef  CONFIG_INPUT_LIGHTNING
#define CONFIG_INPUT_LIGHTNING_MODULE 1
#undef  CONFIG_INPUT_PCIGAME
#define CONFIG_INPUT_PCIGAME_MODULE 1
#undef  CONFIG_INPUT_CS461X
#define CONFIG_INPUT_CS461X_MODULE 1
#undef  CONFIG_INPUT_EMU10K1
#define CONFIG_INPUT_EMU10K1_MODULE 1
#undef  CONFIG_INPUT_SERIO
#define CONFIG_INPUT_SERIO_MODULE 1
#undef  CONFIG_INPUT_SERPORT
#define CONFIG_INPUT_SERPORT_MODULE 1

/*
 * Joysticks
 */
#undef  CONFIG_INPUT_ANALOG
#define CONFIG_INPUT_ANALOG_MODULE 1
#undef  CONFIG_INPUT_A3D
#define CONFIG_INPUT_A3D_MODULE 1
#undef  CONFIG_INPUT_ADI
#define CONFIG_INPUT_ADI_MODULE 1
#undef  CONFIG_INPUT_COBRA
#define CONFIG_INPUT_COBRA_MODULE 1
#undef  CONFIG_INPUT_GF2K
#define CONFIG_INPUT_GF2K_MODULE 1
#undef  CONFIG_INPUT_GRIP
#define CONFIG_INPUT_GRIP_MODULE 1
#undef  CONFIG_INPUT_INTERACT
#define CONFIG_INPUT_INTERACT_MODULE 1
#undef  CONFIG_INPUT_TMDC
#define CONFIG_INPUT_TMDC_MODULE 1
#undef  CONFIG_INPUT_SIDEWINDER
#define CONFIG_INPUT_SIDEWINDER_MODULE 1
#undef  CONFIG_INPUT_IFORCE_USB
#define CONFIG_INPUT_IFORCE_USB_MODULE 1
#undef  CONFIG_INPUT_IFORCE_232
#define CONFIG_INPUT_IFORCE_232_MODULE 1
#undef  CONFIG_INPUT_WARRIOR
#define CONFIG_INPUT_WARRIOR_MODULE 1
#undef  CONFIG_INPUT_MAGELLAN
#define CONFIG_INPUT_MAGELLAN_MODULE 1
#undef  CONFIG_INPUT_SPACEORB
#define CONFIG_INPUT_SPACEORB_MODULE 1
#undef  CONFIG_INPUT_SPACEBALL
#define CONFIG_INPUT_SPACEBALL_MODULE 1
#undef  CONFIG_INPUT_STINGER
#define CONFIG_INPUT_STINGER_MODULE 1
#undef  CONFIG_INPUT_DB9
#define CONFIG_INPUT_DB9_MODULE 1
#undef  CONFIG_INPUT_GAMECON
#define CONFIG_INPUT_GAMECON_MODULE 1
#undef  CONFIG_INPUT_TURBOGRAFX
#define CONFIG_INPUT_TURBOGRAFX_MODULE 1
#undef  CONFIG_QIC02_TAPE
#define CONFIG_QIC02_TAPE_MODULE 1
#undef  CONFIG_QIC02_DYNCONF

/*
 *   Edit configuration parameters in ./include/linux/tpqic02.h!
 */
#undef  CONFIG_IPMI_HANDLER
#define CONFIG_IPMI_HANDLER_MODULE 1
#undef  CONFIG_IPMI_PANIC_EVENT
#undef  CONFIG_IPMI_DEVICE_INTERFACE
#define CONFIG_IPMI_DEVICE_INTERFACE_MODULE 1
#undef  CONFIG_IPMI_KCS
#define CONFIG_IPMI_KCS_MODULE 1
#undef  CONFIG_IPMI_WATCHDOG
#define CONFIG_IPMI_WATCHDOG_MODULE 1

/*
 * Watchdog Cards
 */
#define CONFIG_WATCHDOG 1
#undef  CONFIG_WATCHDOG_NOWAYOUT
#undef  CONFIG_ACQUIRE_WDT
#define CONFIG_ACQUIRE_WDT_MODULE 1
#undef  CONFIG_ADVANTECH_WDT
#define CONFIG_ADVANTECH_WDT_MODULE 1
#undef  CONFIG_ALIM1535_WDT
#define CONFIG_ALIM1535_WDT_MODULE 1
#undef  CONFIG_ALIM7101_WDT
#define CONFIG_ALIM7101_WDT_MODULE 1
#undef  CONFIG_SC520_WDT
#define CONFIG_SC520_WDT_MODULE 1
#undef  CONFIG_PCWATCHDOG
#define CONFIG_PCWATCHDOG_MODULE 1
#undef  CONFIG_EUROTECH_WDT
#define CONFIG_EUROTECH_WDT_MODULE 1
#undef  CONFIG_IB700_WDT
#define CONFIG_IB700_WDT_MODULE 1
#undef  CONFIG_WAFER_WDT
#define CONFIG_WAFER_WDT_MODULE 1
#undef  CONFIG_I810_TCO
#define CONFIG_I810_TCO_MODULE 1
#undef  CONFIG_MIXCOMWD
#define CONFIG_MIXCOMWD_MODULE 1
#undef  CONFIG_60XX_WDT
#define CONFIG_60XX_WDT_MODULE 1
#undef  CONFIG_SC1200_WDT
#define CONFIG_SC1200_WDT_MODULE 1
#undef  CONFIG_SCx200_WDT
#define CONFIG_SCx200_WDT_MODULE 1
#undef  CONFIG_SOFT_WATCHDOG
#define CONFIG_SOFT_WATCHDOG_MODULE 1
#undef  CONFIG_W83877F_WDT
#define CONFIG_W83877F_WDT_MODULE 1
#undef  CONFIG_WDT
#define CONFIG_WDT_MODULE 1
#undef  CONFIG_WDTPCI
#define CONFIG_WDTPCI_MODULE 1
#define CONFIG_WDT_501 1
#define CONFIG_WDT_501_FAN 1
#undef  CONFIG_MACHZ_WDT
#define CONFIG_MACHZ_WDT_MODULE 1
#undef  CONFIG_AMD7XX_TCO
#define CONFIG_AMD7XX_TCO_MODULE 1
#undef  CONFIG_SCx200
#define CONFIG_SCx200_MODULE 1
#undef  CONFIG_SCx200_GPIO
#define CONFIG_SCx200_GPIO_MODULE 1
#undef  CONFIG_AMD_RNG
#define CONFIG_AMD_RNG_MODULE 1
#undef  CONFIG_INTEL_RNG
#define CONFIG_INTEL_RNG_MODULE 1
#undef  CONFIG_HW_RANDOM
#define CONFIG_HW_RANDOM_MODULE 1
#undef  CONFIG_AMD_PM768
#define CONFIG_AMD_PM768_MODULE 1
#undef  CONFIG_NVRAM
#define CONFIG_NVRAM_MODULE 1
#define CONFIG_RTC 1
#undef  CONFIG_DTLK
#define CONFIG_DTLK_MODULE 1
#undef  CONFIG_R3964
#define CONFIG_R3964_MODULE 1
#undef  CONFIG_APPLICOM
#define CONFIG_APPLICOM_MODULE 1
#undef  CONFIG_SONYPI
#define CONFIG_SONYPI_MODULE 1

/*
 * Ftape, the floppy tape device driver
 */
#undef  CONFIG_FTAPE
#define CONFIG_FTAPE_MODULE 1
#undef  CONFIG_ZFTAPE
#define CONFIG_ZFTAPE_MODULE 1
#define CONFIG_ZFT_DFLT_BLK_SZ (10240)

/*
 *   The compressor will be built as a module only!
 */
#undef  CONFIG_ZFT_COMPRESSOR
#define CONFIG_ZFT_COMPRESSOR_MODULE 1
#define CONFIG_FT_NR_BUFFERS (3)
#undef  CONFIG_FT_PROC_FS
#define CONFIG_FT_NORMAL_DEBUG 1
#undef  CONFIG_FT_FULL_DEBUG
#undef  CONFIG_FT_NO_TRACE
#undef  CONFIG_FT_NO_TRACE_AT_ALL

/*
 * Hardware configuration
 */
#define CONFIG_FT_STD_FDC 1
#undef  CONFIG_FT_MACH2
#undef  CONFIG_FT_PROBE_FC10
#undef  CONFIG_FT_ALT_FDC
#define CONFIG_FT_FDC_THR (8)
#define CONFIG_FT_FDC_MAX_RATE (2000)
#define CONFIG_FT_ALPHA_CLOCK (0)
#undef  CONFIG_AGP
#define CONFIG_AGP_MODULE 1
#define CONFIG_AGP_INTEL 1
#define CONFIG_AGP_I810 1
#define CONFIG_AGP_VIA 1
#define CONFIG_AGP_AMD 1
#define CONFIG_AGP_AMD_K8 1
#define CONFIG_AGP_SIS 1
#define CONFIG_AGP_ALI 1
#define CONFIG_AGP_SWORKS 1
#define CONFIG_AGP_NVIDIA 1
#define CONFIG_AGP_ATI 1

/*
 * Direct Rendering Manager (XFree86 DRI support)
 */
#define CONFIG_DRM 1
#undef  CONFIG_DRM_OLD

/*
 * DRM 4.1 drivers
 */
#define CONFIG_DRM_NEW 1
#undef  CONFIG_DRM_TDFX
#define CONFIG_DRM_TDFX_MODULE 1
#undef  CONFIG_DRM_GAMMA
#define CONFIG_DRM_GAMMA_MODULE 1
#undef  CONFIG_DRM_R128
#define CONFIG_DRM_R128_MODULE 1
#undef  CONFIG_DRM_RADEON
#define CONFIG_DRM_RADEON_MODULE 1
#undef  CONFIG_DRM_I810
#define CONFIG_DRM_I810_MODULE 1
#undef  CONFIG_DRM_I810_XFREE_41
#undef  CONFIG_DRM_I830
#define CONFIG_DRM_I830_MODULE 1
#undef  CONFIG_DRM_MGA
#define CONFIG_DRM_MGA_MODULE 1
#undef  CONFIG_DRM_SIS
#define CONFIG_DRM_SIS_MODULE 1

/*
 * PCMCIA character devices
 */
#undef  CONFIG_PCMCIA_SERIAL_CS
#define CONFIG_PCMCIA_SERIAL_CS_MODULE 1
#undef  CONFIG_SYNCLINK_CS
#define CONFIG_SYNCLINK_CS_MODULE 1
#undef  CONFIG_MWAVE
#define CONFIG_MWAVE_MODULE 1
#undef  CONFIG_OBMOUSE
#define CONFIG_OBMOUSE_MODULE 1

/*
 * Multimedia devices
 */
#undef  CONFIG_VIDEO_DEV
#define CONFIG_VIDEO_DEV_MODULE 1

/*
 * Video For Linux
 */
#define CONFIG_VIDEO_PROC_FS 1
#undef  CONFIG_I2C_PARPORT
#define CONFIG_I2C_PARPORT_MODULE 1

/*
 * Video Adapters
 */
#undef  CONFIG_VIDEO_BT848
#define CONFIG_VIDEO_BT848_MODULE 1
#undef  CONFIG_VIDEO_PMS
#define CONFIG_VIDEO_PMS_MODULE 1
#undef  CONFIG_VIDEO_BWQCAM
#define CONFIG_VIDEO_BWQCAM_MODULE 1
#undef  CONFIG_VIDEO_CQCAM
#define CONFIG_VIDEO_CQCAM_MODULE 1
#undef  CONFIG_VIDEO_W9966
#define CONFIG_VIDEO_W9966_MODULE 1
#undef  CONFIG_VIDEO_CPIA
#define CONFIG_VIDEO_CPIA_MODULE 1
#undef  CONFIG_VIDEO_CPIA_PP
#define CONFIG_VIDEO_CPIA_PP_MODULE 1
#undef  CONFIG_VIDEO_CPIA_USB
#define CONFIG_VIDEO_CPIA_USB_MODULE 1
#undef  CONFIG_VIDEO_SAA5249
#define CONFIG_VIDEO_SAA5249_MODULE 1
#undef  CONFIG_TUNER_3036
#define CONFIG_TUNER_3036_MODULE 1
#undef  CONFIG_VIDEO_STRADIS
#define CONFIG_VIDEO_STRADIS_MODULE 1
#undef  CONFIG_VIDEO_ZORAN
#define CONFIG_VIDEO_ZORAN_MODULE 1
#undef  CONFIG_VIDEO_ZORAN_BUZ
#define CONFIG_VIDEO_ZORAN_BUZ_MODULE 1
#undef  CONFIG_VIDEO_ZORAN_DC10
#define CONFIG_VIDEO_ZORAN_DC10_MODULE 1
#undef  CONFIG_VIDEO_ZORAN_LML33
#define CONFIG_VIDEO_ZORAN_LML33_MODULE 1
#undef  CONFIG_VIDEO_ZR36120
#define CONFIG_VIDEO_ZR36120_MODULE 1
#undef  CONFIG_VIDEO_MEYE
#define CONFIG_VIDEO_MEYE_MODULE 1

/*
 * Radio Adapters
 */
#undef  CONFIG_RADIO_CADET
#define CONFIG_RADIO_CADET_MODULE 1
#undef  CONFIG_RADIO_RTRACK
#define CONFIG_RADIO_RTRACK_MODULE 1
#undef  CONFIG_RADIO_RTRACK2
#define CONFIG_RADIO_RTRACK2_MODULE 1
#undef  CONFIG_RADIO_AZTECH
#define CONFIG_RADIO_AZTECH_MODULE 1
#undef  CONFIG_RADIO_GEMTEK
#define CONFIG_RADIO_GEMTEK_MODULE 1
#undef  CONFIG_RADIO_GEMTEK_PCI
#define CONFIG_RADIO_GEMTEK_PCI_MODULE 1
#undef  CONFIG_RADIO_MAXIRADIO
#define CONFIG_RADIO_MAXIRADIO_MODULE 1
#undef  CONFIG_RADIO_MAESTRO
#define CONFIG_RADIO_MAESTRO_MODULE 1
#undef  CONFIG_RADIO_MIROPCM20
#define CONFIG_RADIO_MIROPCM20_MODULE 1
#undef  CONFIG_RADIO_MIROPCM20_RDS
#define CONFIG_RADIO_MIROPCM20_RDS_MODULE 1
#undef  CONFIG_RADIO_SF16FMI
#define CONFIG_RADIO_SF16FMI_MODULE 1
#undef  CONFIG_RADIO_SF16FMR2
#define CONFIG_RADIO_SF16FMR2_MODULE 1
#undef  CONFIG_RADIO_TERRATEC
#define CONFIG_RADIO_TERRATEC_MODULE 1
#undef  CONFIG_RADIO_TRUST
#define CONFIG_RADIO_TRUST_MODULE 1
#undef  CONFIG_RADIO_TYPHOON
#define CONFIG_RADIO_TYPHOON_MODULE 1
#define CONFIG_RADIO_TYPHOON_PROC_FS 1
#undef  CONFIG_RADIO_ZOLTRIX
#define CONFIG_RADIO_ZOLTRIX_MODULE 1

/*
 * File systems
 */
#define CONFIG_QUOTA 1
#undef  CONFIG_QFMT_V2
#define CONFIG_QFMT_V2_MODULE 1
#undef  CONFIG_AUTOFS_FS
#define CONFIG_AUTOFS_FS_MODULE 1
#undef  CONFIG_AUTOFS4_FS
#define CONFIG_AUTOFS4_FS_MODULE 1
#define CONFIG_REISERFS_FS 1
#undef  CONFIG_REISERFS_CHECK
#undef  CONFIG_REISERFS_PROC_INFO
#undef  CONFIG_ADFS_FS
#undef  CONFIG_ADFS_FS_RW
#undef  CONFIG_AFFS_FS
#undef  CONFIG_HFS_FS
#define CONFIG_HFS_FS_MODULE 1
#undef  CONFIG_HFSPLUS_FS
#define CONFIG_HFSPLUS_FS_MODULE 1
#undef  CONFIG_BEFS_FS
#define CONFIG_BEFS_FS_MODULE 1
#undef  CONFIG_BEFS_DEBUG
#undef  CONFIG_BFS_FS
#define CONFIG_EXT3_FS 1
#define CONFIG_JBD 1
#undef  CONFIG_JBD_DEBUG
#define CONFIG_FAT_FS 1
#define CONFIG_MSDOS_FS 1
#define CONFIG_UMSDOS_FS 1
#define CONFIG_VFAT_FS 1
#undef  CONFIG_EFS_FS
#undef  CONFIG_JFFS_FS
#undef  CONFIG_JFFS2_FS
#undef  CONFIG_CRAMFS
#define CONFIG_CRAMFS_MODULE 1
#define CONFIG_TMPFS 1
#define CONFIG_RAMFS 1
#define CONFIG_ISO9660_FS 1
#define CONFIG_JOLIET 1
#define CONFIG_ZISOFS 1
#undef  CONFIG_JFS_FS
#define CONFIG_JFS_FS_MODULE 1
#undef  CONFIG_JFS_DEBUG
#undef  CONFIG_JFS_STATISTICS
#undef  CONFIG_MINIX_FS
#define CONFIG_MINIX_FS_MODULE 1
#undef  CONFIG_VXFS_FS
#define CONFIG_VXFS_FS_MODULE 1
#undef  CONFIG_NTFS_FS
#define CONFIG_NTFS_FS_MODULE 1
#undef  CONFIG_NTFS_RW
#undef  CONFIG_HPFS_FS
#define CONFIG_HPFS_FS_MODULE 1
#define CONFIG_PROC_FS 1
#undef  CONFIG_DEVFS_FS
#undef  CONFIG_DEVFS_MOUNT
#undef  CONFIG_DEVFS_DEBUG
#define CONFIG_DEVPTS_FS 1
#undef  CONFIG_QNX4FS_FS
#define CONFIG_QNX4FS_FS_MODULE 1
#undef  CONFIG_QNX4FS_RW
#undef  CONFIG_ROMFS_FS
#define CONFIG_EXT2_FS 1
#undef  CONFIG_SYSV_FS
#define CONFIG_SYSV_FS_MODULE 1
#undef  CONFIG_UDF_FS
#define CONFIG_UDF_FS_MODULE 1
#undef  CONFIG_UDF_RW
#undef  CONFIG_UFS_FS
#define CONFIG_UFS_FS_MODULE 1
#define CONFIG_UFS_FS_WRITE 1
#undef  CONFIG_XFS_FS
#define CONFIG_XFS_FS_MODULE 1
#define CONFIG_XFS_QUOTA 1
#undef  CONFIG_XFS_RT
#undef  CONFIG_XFS_TRACE
#undef  CONFIG_XFS_DEBUG

/*
 * Network File Systems
 */
#undef  CONFIG_CODA_FS
#define CONFIG_CODA_FS_MODULE 1
#undef  CONFIG_INTERMEZZO_FS
#define CONFIG_INTERMEZZO_FS_MODULE 1
#define CONFIG_NFS_FS 1
#define CONFIG_NFS_V3 1
#undef  CONFIG_NFS_DIRECTIO
#undef  CONFIG_ROOT_NFS
#undef  CONFIG_NFSD
#define CONFIG_NFSD_MODULE 1
#define CONFIG_NFSD_V3 1
#undef  CONFIG_NFSD_TCP
#define CONFIG_SUNRPC 1
#define CONFIG_LOCKD 1
#define CONFIG_LOCKD_V4 1
#undef  CONFIG_SMB_FS
#define CONFIG_SMB_FS_MODULE 1
#undef  CONFIG_SMB_NLS_DEFAULT
#define CONFIG_SMB_UNIX 1
#undef  CONFIG_NCP_FS
#define CONFIG_NCP_FS_MODULE 1
#undef  CONFIG_NCPFS_PACKET_SIGNING
#undef  CONFIG_NCPFS_IOCTL_LOCKING
#define CONFIG_NCPFS_STRONG 1
#define CONFIG_NCPFS_NFS_NS 1
#define CONFIG_NCPFS_OS2_NS 1
#undef  CONFIG_NCPFS_SMALLDOS
#undef  CONFIG_NCPFS_NLS
#undef  CONFIG_NCPFS_EXTRAS
#define CONFIG_ZISOFS_FS 1

/*
 * Partition Types
 */
#define CONFIG_PARTITION_ADVANCED 1
#undef  CONFIG_ACORN_PARTITION
#undef  CONFIG_OSF_PARTITION
#undef  CONFIG_AMIGA_PARTITION
#undef  CONFIG_ATARI_PARTITION
#define CONFIG_MAC_PARTITION 1
#define CONFIG_MSDOS_PARTITION 1
#define CONFIG_BSD_DISKLABEL 1
#undef  CONFIG_MINIX_SUBPARTITION
#undef  CONFIG_SOLARIS_X86_PARTITION
#undef  CONFIG_UNIXWARE_DISKLABEL
#define CONFIG_LDM_PARTITION 1
#undef  CONFIG_LDM_DEBUG
#undef  CONFIG_SGI_PARTITION
#undef  CONFIG_ULTRIX_PARTITION
#undef  CONFIG_SUN_PARTITION
#undef  CONFIG_EFI_PARTITION
#define CONFIG_SMB_NLS 1
#define CONFIG_NLS 1

/*
 * Native Language Support
 */
#define CONFIG_NLS_DEFAULT "cp437"
#define CONFIG_NLS_CODEPAGE_437 1
#undef  CONFIG_NLS_CODEPAGE_737
#define CONFIG_NLS_CODEPAGE_737_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_775
#define CONFIG_NLS_CODEPAGE_775_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_850
#define CONFIG_NLS_CODEPAGE_850_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_852
#define CONFIG_NLS_CODEPAGE_852_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_855
#define CONFIG_NLS_CODEPAGE_855_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_857
#define CONFIG_NLS_CODEPAGE_857_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_860
#define CONFIG_NLS_CODEPAGE_860_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_861
#define CONFIG_NLS_CODEPAGE_861_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_862
#define CONFIG_NLS_CODEPAGE_862_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_863
#define CONFIG_NLS_CODEPAGE_863_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_864
#define CONFIG_NLS_CODEPAGE_864_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_865
#define CONFIG_NLS_CODEPAGE_865_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_866
#define CONFIG_NLS_CODEPAGE_866_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_869
#define CONFIG_NLS_CODEPAGE_869_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_936
#define CONFIG_NLS_CODEPAGE_936_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_950
#define CONFIG_NLS_CODEPAGE_950_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_932
#define CONFIG_NLS_CODEPAGE_932_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_949
#define CONFIG_NLS_CODEPAGE_949_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_874
#define CONFIG_NLS_CODEPAGE_874_MODULE 1
#undef  CONFIG_NLS_ISO8859_8
#define CONFIG_NLS_ISO8859_8_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_1250
#define CONFIG_NLS_CODEPAGE_1250_MODULE 1
#undef  CONFIG_NLS_CODEPAGE_1251
#define CONFIG_NLS_CODEPAGE_1251_MODULE 1
#define CONFIG_NLS_ISO8859_1 1
#undef  CONFIG_NLS_ISO8859_2
#define CONFIG_NLS_ISO8859_2_MODULE 1
#undef  CONFIG_NLS_ISO8859_3
#define CONFIG_NLS_ISO8859_3_MODULE 1
#undef  CONFIG_NLS_ISO8859_4
#define CONFIG_NLS_ISO8859_4_MODULE 1
#undef  CONFIG_NLS_ISO8859_5
#define CONFIG_NLS_ISO8859_5_MODULE 1
#undef  CONFIG_NLS_ISO8859_6
#define CONFIG_NLS_ISO8859_6_MODULE 1
#undef  CONFIG_NLS_ISO8859_7
#define CONFIG_NLS_ISO8859_7_MODULE 1
#undef  CONFIG_NLS_ISO8859_9
#define CONFIG_NLS_ISO8859_9_MODULE 1
#undef  CONFIG_NLS_ISO8859_13
#define CONFIG_NLS_ISO8859_13_MODULE 1
#undef  CONFIG_NLS_ISO8859_14
#define CONFIG_NLS_ISO8859_14_MODULE 1
#undef  CONFIG_NLS_ISO8859_15
#define CONFIG_NLS_ISO8859_15_MODULE 1
#undef  CONFIG_NLS_KOI8_R
#define CONFIG_NLS_KOI8_R_MODULE 1
#undef  CONFIG_NLS_KOI8_U
#define CONFIG_NLS_KOI8_U_MODULE 1
#undef  CONFIG_NLS_UTF8
#define CONFIG_NLS_UTF8_MODULE 1

/*
 * Console drivers
 */
#define CONFIG_VGA_CONSOLE 1
#define CONFIG_VIDEO_SELECT 1
#undef  CONFIG_MDA_CONSOLE

/*
 * Frame-buffer support
 */
#define CONFIG_FB 1
#define CONFIG_DUMMY_CONSOLE 1
#undef  CONFIG_FB_RIVA
#define CONFIG_FB_RIVA_MODULE 1
#undef  CONFIG_FB_CLGEN
#define CONFIG_FB_CLGEN_MODULE 1
#undef  CONFIG_FB_PM2
#define CONFIG_FB_PM2_MODULE 1
#undef  CONFIG_FB_PM2_FIFO_DISCONNECT
#define CONFIG_FB_PM2_PCI 1
#undef  CONFIG_FB_PM3
#define CONFIG_FB_PM3_MODULE 1
#undef  CONFIG_FB_CYBER2000
#define CONFIG_FB_CYBER2000_MODULE 1
#define CONFIG_FB_VESA 1
#undef  CONFIG_FB_VGA16
#define CONFIG_FB_VGA16_MODULE 1
#undef  CONFIG_FB_HGA
#define CONFIG_FB_HGA_MODULE 1
#define CONFIG_VIDEO_SELECT 1
#undef  CONFIG_FB_MATROX
#define CONFIG_FB_MATROX_MODULE 1
#define CONFIG_FB_MATROX_MILLENIUM 1
#define CONFIG_FB_MATROX_MYSTIQUE 1
#define CONFIG_FB_MATROX_G450 1
#define CONFIG_FB_MATROX_G100 1
#undef  CONFIG_FB_MATROX_I2C
#define CONFIG_FB_MATROX_I2C_MODULE 1
#undef  CONFIG_FB_MATROX_MAVEN
#define CONFIG_FB_MATROX_MAVEN_MODULE 1
#undef  CONFIG_FB_MATROX_PROC
#define CONFIG_FB_MATROX_PROC_MODULE 1
#define CONFIG_FB_MATROX_MULTIHEAD 1
#undef  CONFIG_FB_ATY
#define CONFIG_FB_ATY_MODULE 1
#define CONFIG_FB_ATY_GX 1
#define CONFIG_FB_ATY_CT 1
#define CONFIG_FB_ATY_GENERIC_LCD 1
#undef  CONFIG_FB_RADEON
#define CONFIG_FB_RADEON_MODULE 1
#undef  CONFIG_FB_ATY128
#define CONFIG_FB_ATY128_MODULE 1
#undef  CONFIG_FB_INTEL
#define CONFIG_FB_INTEL_MODULE 1
#undef  CONFIG_FB_SIS
#define CONFIG_FB_SIS_MODULE 1
#define CONFIG_FB_SIS_300 1
#define CONFIG_FB_SIS_315 1
#undef  CONFIG_FB_NEOMAGIC
#define CONFIG_FB_NEOMAGIC_MODULE 1
#undef  CONFIG_FB_3DFX
#define CONFIG_FB_3DFX_MODULE 1
#undef  CONFIG_FB_VOODOO1
#define CONFIG_FB_VOODOO1_MODULE 1
#undef  CONFIG_FB_TRIDENT
#define CONFIG_FB_TRIDENT_MODULE 1
#undef  CONFIG_FB_IT8181
#define CONFIG_FB_IT8181_MODULE 1
#undef  CONFIG_FB_VIRTUAL
#define CONFIG_FBCON_ADVANCED 1
#undef  CONFIG_FBCON_MFB
#define CONFIG_FBCON_MFB_MODULE 1
#undef  CONFIG_FBCON_CFB2
#define CONFIG_FBCON_CFB2_MODULE 1
#undef  CONFIG_FBCON_CFB4
#define CONFIG_FBCON_CFB4_MODULE 1
#define CONFIG_FBCON_CFB8 1
#define CONFIG_FBCON_CFB16 1
#define CONFIG_FBCON_CFB24 1
#define CONFIG_FBCON_CFB32 1
#undef  CONFIG_FBCON_AFB
#undef  CONFIG_FBCON_ILBM
#undef  CONFIG_FBCON_IPLAN2P2
#undef  CONFIG_FBCON_IPLAN2P4
#undef  CONFIG_FBCON_IPLAN2P8
#undef  CONFIG_FBCON_MAC
#undef  CONFIG_FBCON_VGA_PLANES
#define CONFIG_FBCON_VGA 1
#undef  CONFIG_FBCON_HGA
#define CONFIG_FBCON_HGA_MODULE 1
#undef  CONFIG_FBCON_FONTWIDTH8_ONLY
#define CONFIG_FBCON_FONTS 1
#define CONFIG_FONT_8x8 1
#define CONFIG_FONT_8x16 1
#undef  CONFIG_FONT_SUN8x16
#undef  CONFIG_FONT_SUN12x22
#undef  CONFIG_FONT_6x11
#undef  CONFIG_FONT_PEARL_8x8
#undef  CONFIG_FONT_ACORN_8x8

/*
 * Sound
 */
#undef  CONFIG_SOUND
#define CONFIG_SOUND_MODULE 1
#undef  CONFIG_SOUND_ALI5455
#define CONFIG_SOUND_ALI5455_MODULE 1
#define CONFIG_SOUND_ALI5455_CODECSPDIFOUT_PCMOUTSHARE 1
#define CONFIG_SOUND_ALI5455_CODECSPDIFOUT_CODECINDEPENDENTDMA 1
#define CONFIG_SOUND_ALI5455_CONTROLLERSPDIFOUT_PCMOUTSHARE 1
#define CONFIG_SOUND_ALI5455_CONTROLLERSPDIFOUT_CONTROLLERINDEPENDENTDMA 1
#undef  CONFIG_SOUND_BT878
#define CONFIG_SOUND_BT878_MODULE 1
#undef  CONFIG_SOUND_CMPCI
#define CONFIG_SOUND_CMPCI_MODULE 1
#undef  CONFIG_SOUND_CMPCI_FM
#undef  CONFIG_SOUND_CMPCI_MIDI
#define CONFIG_SOUND_CMPCI_JOYSTICK 1
#define CONFIG_SOUND_CMPCI_CM8738 1
#undef  CONFIG_SOUND_CMPCI_SPDIFINVERSE
#define CONFIG_SOUND_CMPCI_SPDIFLOOP 1
#define CONFIG_SOUND_CMPCI_SPEAKERS (2)
#undef  CONFIG_SOUND_EMU10K1
#define CONFIG_SOUND_EMU10K1_MODULE 1
#define CONFIG_MIDI_EMU10K1 1
#undef  CONFIG_SOUND_FUSION
#define CONFIG_SOUND_FUSION_MODULE 1
#undef  CONFIG_SOUND_CS4281
#define CONFIG_SOUND_CS4281_MODULE 1
#undef  CONFIG_SOUND_ES1370
#define CONFIG_SOUND_ES1370_MODULE 1
#undef  CONFIG_SOUND_ES1371
#define CONFIG_SOUND_ES1371_MODULE 1
#undef  CONFIG_SOUND_ESSSOLO1
#define CONFIG_SOUND_ESSSOLO1_MODULE 1
#undef  CONFIG_SOUND_MAESTRO
#define CONFIG_SOUND_MAESTRO_MODULE 1
#undef  CONFIG_SOUND_MAESTRO3
#define CONFIG_SOUND_MAESTRO3_MODULE 1
#undef  CONFIG_SOUND_FORTE
#define CONFIG_SOUND_FORTE_MODULE 1
#undef  CONFIG_SOUND_ICH
#define CONFIG_SOUND_ICH_MODULE 1
#undef  CONFIG_SOUND_RME96XX
#define CONFIG_SOUND_RME96XX_MODULE 1
#undef  CONFIG_SOUND_SONICVIBES
#define CONFIG_SOUND_SONICVIBES_MODULE 1
#undef  CONFIG_SOUND_TRIDENT
#define CONFIG_SOUND_TRIDENT_MODULE 1
#undef  CONFIG_SOUND_MSNDCLAS
#define CONFIG_SOUND_MSNDCLAS_MODULE 1
#undef  CONFIG_MSNDCLAS_HAVE_BOOT
#define CONFIG_MSNDCLAS_INIT_FILE "/etc/sound/msndinit.bin"
#define CONFIG_MSNDCLAS_PERM_FILE "/etc/sound/msndperm.bin"
#undef  CONFIG_SOUND_MSNDPIN
#define CONFIG_SOUND_MSNDPIN_MODULE 1
#undef  CONFIG_MSNDPIN_HAVE_BOOT
#define CONFIG_MSNDPIN_INIT_FILE "/etc/sound/pndspini.bin"
#define CONFIG_MSNDPIN_PERM_FILE "/etc/sound/pndsperm.bin"
#undef  CONFIG_SOUND_VIA82CXXX
#define CONFIG_SOUND_VIA82CXXX_MODULE 1
#undef  CONFIG_MIDI_VIA82CXXX
#undef  CONFIG_SOUND_OSS
#define CONFIG_SOUND_OSS_MODULE 1
#undef  CONFIG_SOUND_TRACEINIT
#undef  CONFIG_SOUND_DMAP
#undef  CONFIG_SOUND_AD1816
#define CONFIG_SOUND_AD1816_MODULE 1
#undef  CONFIG_SOUND_AD1889
#define CONFIG_SOUND_AD1889_MODULE 1
#undef  CONFIG_SOUND_SGALAXY
#define CONFIG_SOUND_SGALAXY_MODULE 1
#undef  CONFIG_SOUND_ADLIB
#define CONFIG_SOUND_ADLIB_MODULE 1
#undef  CONFIG_SOUND_ACI_MIXER
#define CONFIG_SOUND_ACI_MIXER_MODULE 1
#undef  CONFIG_SOUND_CS4232
#define CONFIG_SOUND_CS4232_MODULE 1
#undef  CONFIG_SOUND_SSCAPE
#define CONFIG_SOUND_SSCAPE_MODULE 1
#undef  CONFIG_SOUND_GUS
#define CONFIG_SOUND_GUS_MODULE 1
#undef  CONFIG_SOUND_GUS16
#undef  CONFIG_SOUND_GUSMAX
#undef  CONFIG_SOUND_VMIDI
#define CONFIG_SOUND_VMIDI_MODULE 1
#undef  CONFIG_SOUND_TRIX
#define CONFIG_SOUND_TRIX_MODULE 1
#undef  CONFIG_SOUND_MSS
#define CONFIG_SOUND_MSS_MODULE 1
#undef  CONFIG_SOUND_MPU401
#define CONFIG_SOUND_MPU401_MODULE 1
#undef  CONFIG_SOUND_NM256
#define CONFIG_SOUND_NM256_MODULE 1
#undef  CONFIG_SOUND_MAD16
#define CONFIG_SOUND_MAD16_MODULE 1
#undef  CONFIG_MAD16_OLDCARD
#undef  CONFIG_SOUND_PAS
#define CONFIG_SOUND_PAS_MODULE 1
#undef  CONFIG_PAS_JOYSTICK
#undef  CONFIG_SOUND_PSS
#define CONFIG_SOUND_PSS_MODULE 1
#undef  CONFIG_PSS_MIXER
#undef  CONFIG_PSS_HAVE_BOOT
#undef  CONFIG_SOUND_SB
#define CONFIG_SOUND_SB_MODULE 1
#undef  CONFIG_SOUND_AWE32_SYNTH
#define CONFIG_SOUND_AWE32_SYNTH_MODULE 1
#undef  CONFIG_SOUND_KAHLUA
#define CONFIG_SOUND_KAHLUA_MODULE 1
#undef  CONFIG_SOUND_WAVEFRONT
#define CONFIG_SOUND_WAVEFRONT_MODULE 1
#undef  CONFIG_SOUND_MAUI
#define CONFIG_SOUND_MAUI_MODULE 1
#undef  CONFIG_SOUND_YM3812
#define CONFIG_SOUND_YM3812_MODULE 1
#undef  CONFIG_SOUND_OPL3SA1
#define CONFIG_SOUND_OPL3SA1_MODULE 1
#undef  CONFIG_SOUND_OPL3SA2
#define CONFIG_SOUND_OPL3SA2_MODULE 1
#undef  CONFIG_SOUND_YMFPCI
#define CONFIG_SOUND_YMFPCI_MODULE 1
#undef  CONFIG_SOUND_YMFPCI_LEGACY
#undef  CONFIG_SOUND_UART6850
#define CONFIG_SOUND_UART6850_MODULE 1
#undef  CONFIG_SOUND_AEDSP16
#define CONFIG_SOUND_AEDSP16_MODULE 1
#define CONFIG_SC6600 1
#define CONFIG_SC6600_JOY 1
#define CONFIG_SC6600_CDROM (4)
#define CONFIG_SC6600_CDROMBASE 0x0
#define CONFIG_AEDSP16_SBPRO 1
#define CONFIG_AEDSP16_MPU401 1
#undef  CONFIG_SOUND_TVMIXER
#define CONFIG_SOUND_TVMIXER_MODULE 1
#undef  CONFIG_SOUND_AD1980
#define CONFIG_SOUND_AD1980_MODULE 1
#undef  CONFIG_SOUND_WM97XX
#define CONFIG_SOUND_WM97XX_MODULE 1

/*
 * USB support
 */
#undef  CONFIG_USB
#define CONFIG_USB_MODULE 1
#undef  CONFIG_USB_DEBUG

/*
 * Miscellaneous USB options
 */
#define CONFIG_USB_DEVICEFS 1
#undef  CONFIG_USB_BANDWIDTH

/*
 * USB Host Controller Drivers
 */
#undef  CONFIG_USB_EHCI_HCD
#define CONFIG_USB_EHCI_HCD_MODULE 1
#undef  CONFIG_USB_UHCI
#define CONFIG_USB_UHCI_MODULE 1
#undef  CONFIG_USB_UHCI_ALT
#define CONFIG_USB_UHCI_ALT_MODULE 1
#undef  CONFIG_USB_OHCI
#define CONFIG_USB_OHCI_MODULE 1
#undef  CONFIG_USB_SL811HS_ALT
#define CONFIG_USB_SL811HS_ALT_MODULE 1
#undef  CONFIG_USB_SL811HS
#define CONFIG_USB_SL811HS_MODULE 1

/*
 * USB Device Class drivers
 */
#undef  CONFIG_USB_AUDIO
#define CONFIG_USB_AUDIO_MODULE 1
#undef  CONFIG_USB_EMI26
#define CONFIG_USB_EMI26_MODULE 1

/*
 *   USB Bluetooth can only be used with disabled Bluetooth subsystem
 */
#undef  CONFIG_USB_MIDI
#define CONFIG_USB_MIDI_MODULE 1
#undef  CONFIG_USB_STORAGE
#define CONFIG_USB_STORAGE_MODULE 1
#undef  CONFIG_USB_STORAGE_DEBUG
#define CONFIG_USB_STORAGE_DATAFAB 1
#define CONFIG_USB_STORAGE_FREECOM 1
#define CONFIG_USB_STORAGE_ISD200 1
#define CONFIG_USB_STORAGE_DPCM 1
#define CONFIG_USB_STORAGE_HP8200e 1
#define CONFIG_USB_STORAGE_SDDR09 1
#define CONFIG_USB_STORAGE_SDDR55 1
#define CONFIG_USB_STORAGE_JUMPSHOT 1
#undef  CONFIG_USB_ACM
#define CONFIG_USB_ACM_MODULE 1
#undef  CONFIG_USB_PRINTER
#define CONFIG_USB_PRINTER_MODULE 1

/*
 * USB Human Interface Devices (HID)
 */
#undef  CONFIG_USB_HID
#define CONFIG_USB_HID_MODULE 1
#define CONFIG_USB_HIDINPUT 1
#define CONFIG_USB_HIDDEV 1
#undef  CONFIG_USB_KBD
#define CONFIG_USB_KBD_MODULE 1
#undef  CONFIG_USB_MOUSE
#define CONFIG_USB_MOUSE_MODULE 1
#undef  CONFIG_USB_AIPTEK
#define CONFIG_USB_AIPTEK_MODULE 1
#undef  CONFIG_USB_WACOM
#define CONFIG_USB_WACOM_MODULE 1
#undef  CONFIG_USB_KBTAB
#define CONFIG_USB_KBTAB_MODULE 1
#undef  CONFIG_USB_POWERMATE
#define CONFIG_USB_POWERMATE_MODULE 1

/*
 * USB Imaging devices
 */
#undef  CONFIG_USB_DC2XX
#define CONFIG_USB_DC2XX_MODULE 1
#undef  CONFIG_USB_MDC800
#define CONFIG_USB_MDC800_MODULE 1
#undef  CONFIG_USB_SCANNER
#define CONFIG_USB_SCANNER_MODULE 1
#undef  CONFIG_USB_MICROTEK
#define CONFIG_USB_MICROTEK_MODULE 1
#undef  CONFIG_USB_HPUSBSCSI
#define CONFIG_USB_HPUSBSCSI_MODULE 1

/*
 * USB Multimedia devices
 */
#undef  CONFIG_USB_IBMCAM
#define CONFIG_USB_IBMCAM_MODULE 1
#undef  CONFIG_USB_KONICAWC
#define CONFIG_USB_KONICAWC_MODULE 1
#undef  CONFIG_USB_OV511
#define CONFIG_USB_OV511_MODULE 1
#undef  CONFIG_USB_PWC
#define CONFIG_USB_PWC_MODULE 1
#undef  CONFIG_USB_SE401
#define CONFIG_USB_SE401_MODULE 1
#undef  CONFIG_USB_STV680
#define CONFIG_USB_STV680_MODULE 1
#undef  CONFIG_USB_W9968CF
#define CONFIG_USB_W9968CF_MODULE 1
#undef  CONFIG_USB_VICAM
#define CONFIG_USB_VICAM_MODULE 1
#undef  CONFIG_USB_DSBR
#define CONFIG_USB_DSBR_MODULE 1
#undef  CONFIG_USB_DABUSB
#define CONFIG_USB_DABUSB_MODULE 1

/*
 * USB Network adaptors
 */
#undef  CONFIG_USB_PEGASUS
#define CONFIG_USB_PEGASUS_MODULE 1
#undef  CONFIG_USB_RTL8150
#define CONFIG_USB_RTL8150_MODULE 1
#undef  CONFIG_USB_KAWETH
#define CONFIG_USB_KAWETH_MODULE 1
#undef  CONFIG_USB_CATC
#define CONFIG_USB_CATC_MODULE 1
#undef  CONFIG_USB_CDCETHER
#define CONFIG_USB_CDCETHER_MODULE 1
#undef  CONFIG_USB_USBNET
#define CONFIG_USB_USBNET_MODULE 1

/*
 * USB port drivers
 */
#undef  CONFIG_USB_USS720
#define CONFIG_USB_USS720_MODULE 1

/*
 * USB Serial Converter support
 */
#undef  CONFIG_USB_SERIAL
#define CONFIG_USB_SERIAL_MODULE 1
#undef  CONFIG_USB_SERIAL_DEBUG
#define CONFIG_USB_SERIAL_GENERIC 1
#undef  CONFIG_USB_SERIAL_BELKIN
#define CONFIG_USB_SERIAL_BELKIN_MODULE 1
#undef  CONFIG_USB_SERIAL_WHITEHEAT
#define CONFIG_USB_SERIAL_WHITEHEAT_MODULE 1
#undef  CONFIG_USB_SERIAL_DIGI_ACCELEPORT
#define CONFIG_USB_SERIAL_DIGI_ACCELEPORT_MODULE 1
#undef  CONFIG_USB_SERIAL_EMPEG
#define CONFIG_USB_SERIAL_EMPEG_MODULE 1
#undef  CONFIG_USB_SERIAL_FTDI_SIO
#define CONFIG_USB_SERIAL_FTDI_SIO_MODULE 1
#undef  CONFIG_USB_SERIAL_VISOR
#define CONFIG_USB_SERIAL_VISOR_MODULE 1
#undef  CONFIG_USB_SERIAL_IPAQ
#define CONFIG_USB_SERIAL_IPAQ_MODULE 1
#undef  CONFIG_USB_SERIAL_IR
#define CONFIG_USB_SERIAL_IR_MODULE 1
#undef  CONFIG_USB_SERIAL_EDGEPORT
#define CONFIG_USB_SERIAL_EDGEPORT_MODULE 1
#undef  CONFIG_USB_SERIAL_EDGEPORT_TI
#define CONFIG_USB_SERIAL_EDGEPORT_TI_MODULE 1
#undef  CONFIG_USB_SERIAL_KEYSPAN_PDA
#define CONFIG_USB_SERIAL_KEYSPAN_PDA_MODULE 1
#undef  CONFIG_USB_SERIAL_KEYSPAN
#define CONFIG_USB_SERIAL_KEYSPAN_MODULE 1
#define CONFIG_USB_SERIAL_KEYSPAN_USA28 1
#define CONFIG_USB_SERIAL_KEYSPAN_USA28X 1
#define CONFIG_USB_SERIAL_KEYSPAN_USA28XA 1
#define CONFIG_USB_SERIAL_KEYSPAN_USA28XB 1
#define CONFIG_USB_SERIAL_KEYSPAN_USA19 1
#define CONFIG_USB_SERIAL_KEYSPAN_USA18X 1
#define CONFIG_USB_SERIAL_KEYSPAN_USA19W 1
#define CONFIG_USB_SERIAL_KEYSPAN_USA19QW 1
#define CONFIG_USB_SERIAL_KEYSPAN_USA19QI 1
#define CONFIG_USB_SERIAL_KEYSPAN_MPR 1
#define CONFIG_USB_SERIAL_KEYSPAN_USA49W 1
#define CONFIG_USB_SERIAL_KEYSPAN_USA49WLC 1
#undef  CONFIG_USB_SERIAL_MCT_U232
#define CONFIG_USB_SERIAL_MCT_U232_MODULE 1
#undef  CONFIG_USB_SERIAL_KLSI
#define CONFIG_USB_SERIAL_KLSI_MODULE 1
#undef  CONFIG_USB_SERIAL_KOBIL_SCT
#define CONFIG_USB_SERIAL_KOBIL_SCT_MODULE 1
#undef  CONFIG_USB_SERIAL_PL2303
#define CONFIG_USB_SERIAL_PL2303_MODULE 1
#undef  CONFIG_USB_SERIAL_CYBERJACK
#define CONFIG_USB_SERIAL_CYBERJACK_MODULE 1
#undef  CONFIG_USB_SERIAL_XIRCOM
#define CONFIG_USB_SERIAL_XIRCOM_MODULE 1
#undef  CONFIG_USB_SERIAL_OMNINET
#define CONFIG_USB_SERIAL_OMNINET_MODULE 1

/*
 * USB Miscellaneous drivers
 */
#undef  CONFIG_USB_RIO500
#define CONFIG_USB_RIO500_MODULE 1
#undef  CONFIG_USB_AUERSWALD
#define CONFIG_USB_AUERSWALD_MODULE 1
#undef  CONFIG_USB_TIGL
#define CONFIG_USB_TIGL_MODULE 1
#undef  CONFIG_USB_BRLVGER
#define CONFIG_USB_BRLVGER_MODULE 1
#undef  CONFIG_USB_LCD
#define CONFIG_USB_LCD_MODULE 1

/*
 * Support for USB gadgets
 */
#undef  CONFIG_USB_GADGET

/*
 * Bluetooth support
 */
#undef  CONFIG_BLUEZ
#define CONFIG_BLUEZ_MODULE 1
#undef  CONFIG_BLUEZ_L2CAP
#define CONFIG_BLUEZ_L2CAP_MODULE 1
#undef  CONFIG_BLUEZ_SCO
#define CONFIG_BLUEZ_SCO_MODULE 1
#undef  CONFIG_BLUEZ_RFCOMM
#define CONFIG_BLUEZ_RFCOMM_MODULE 1
#define CONFIG_BLUEZ_RFCOMM_TTY 1
#undef  CONFIG_BLUEZ_BNEP
#define CONFIG_BLUEZ_BNEP_MODULE 1
#define CONFIG_BLUEZ_BNEP_MC_FILTER 1
#define CONFIG_BLUEZ_BNEP_PROTO_FILTER 1
#undef  CONFIG_BLUEZ_CMTP
#define CONFIG_BLUEZ_CMTP_MODULE 1

/*
 * Bluetooth device drivers
 */
#undef  CONFIG_BLUEZ_HCIUSB
#define CONFIG_BLUEZ_HCIUSB_MODULE 1
#define CONFIG_BLUEZ_HCIUSB_SCO 1
#undef  CONFIG_BLUEZ_HCIUART
#define CONFIG_BLUEZ_HCIUART_MODULE 1
#define CONFIG_BLUEZ_HCIUART_H4 1
#define CONFIG_BLUEZ_HCIUART_BCSP 1
#define CONFIG_BLUEZ_HCIUART_BCSP_TXCRC 1
#undef  CONFIG_BLUEZ_HCIBFUSB
#define CONFIG_BLUEZ_HCIBFUSB_MODULE 1
#undef  CONFIG_BLUEZ_HCIDTL1
#define CONFIG_BLUEZ_HCIDTL1_MODULE 1
#undef  CONFIG_BLUEZ_HCIBT3C
#define CONFIG_BLUEZ_HCIBT3C_MODULE 1
#undef  CONFIG_BLUEZ_HCIBLUECARD
#define CONFIG_BLUEZ_HCIBLUECARD_MODULE 1
#undef  CONFIG_BLUEZ_HCIBTUART
#define CONFIG_BLUEZ_HCIBTUART_MODULE 1
#undef  CONFIG_BLUEZ_HCIVHCI
#define CONFIG_BLUEZ_HCIVHCI_MODULE 1

/*
 * Kernel hacking
 */
#undef  CONFIG_DEBUG_KERNEL
#define CONFIG_LOG_BUF_SHIFT (0)

/*
 * Cryptographic options
 */
#define CONFIG_CRYPTO 1
#define CONFIG_CRYPTO_HMAC 1
#undef  CONFIG_CRYPTO_NULL
#define CONFIG_CRYPTO_NULL_MODULE 1
#undef  CONFIG_CRYPTO_MD4
#define CONFIG_CRYPTO_MD4_MODULE 1
#undef  CONFIG_CRYPTO_MD5
#define CONFIG_CRYPTO_MD5_MODULE 1
#undef  CONFIG_CRYPTO_SHA1
#define CONFIG_CRYPTO_SHA1_MODULE 1
#undef  CONFIG_CRYPTO_SHA256
#define CONFIG_CRYPTO_SHA256_MODULE 1
#undef  CONFIG_CRYPTO_SHA512
#define CONFIG_CRYPTO_SHA512_MODULE 1
#undef  CONFIG_CRYPTO_DES
#define CONFIG_CRYPTO_DES_MODULE 1
#undef  CONFIG_CRYPTO_BLOWFISH
#define CONFIG_CRYPTO_BLOWFISH_MODULE 1
#undef  CONFIG_CRYPTO_TWOFISH
#define CONFIG_CRYPTO_TWOFISH_MODULE 1
#undef  CONFIG_CRYPTO_SERPENT
#define CONFIG_CRYPTO_SERPENT_MODULE 1
#undef  CONFIG_CRYPTO_AES
#define CONFIG_CRYPTO_AES_MODULE 1
#undef  CONFIG_CRYPTO_CAST5
#define CONFIG_CRYPTO_CAST5_MODULE 1
#undef  CONFIG_CRYPTO_CAST6
#define CONFIG_CRYPTO_CAST6_MODULE 1
#undef  CONFIG_CRYPTO_ARC4
#define CONFIG_CRYPTO_ARC4_MODULE 1
#undef  CONFIG_CRYPTO_DEFLATE
#define CONFIG_CRYPTO_DEFLATE_MODULE 1
#undef  CONFIG_CRYPTO_TEST
#define CONFIG_CRYPTO_TEST_MODULE 1

/*
 * Library routines
 */
#undef  CONFIG_CRC32
#define CONFIG_CRC32_MODULE 1
#define CONFIG_ZLIB_INFLATE 1
#undef  CONFIG_ZLIB_DEFLATE
#define CONFIG_ZLIB_DEFLATE_MODULE 1
#undef  CONFIG_FW_LOADER
#define CONFIG_FW_LOADER_MODULE 1
